#include"oat.hpp"


using namespace oat;


namespace{


const char*  const filename = "__aicone.txt";


enum{
  pixel_size = 8,

  l_max = 7,

  draw_point_flag = 1,
  draw_line_flag  = 2,
  fill_area_flag  = 4,

};


Area*  drawing_area;


uint16_t  code;


Icon  icon;
Icon   tmp;


int  first_x = -1;
int  first_y     ;


void
draw_line(int  x0, int  y0, int  x1, int  y1, uint16_t  pixel)
{
  int  dx = std::abs(x1-x0);
  int  dy = std::abs(y1-y0);

  const int  sx = ((x0 < x1)? 1:-1);
  const int  sy = ((y0 < y1)? 1:-1);

  int  err = (dx-dy);

    for(;;)
    {
      icon.data[y0][x0] = pixel;

        if((x0 == x1) && (y0 == y1))
        {
          break;
        }


      const int  e2 = (2*err);

        if(e2 > -dy)
        {
          err = err-dy;
          x0  = x0 +sx;
        }


        if(e2 < dx)
        {
          err = err+dx;
          y0  = y0 +sy; 
        }
    }
}


void
fill_area(int  x, int  y, uint16_t  a, uint16_t  b)
{
    if(icon.data[y][x] == a)
    {
      icon.data[y][x] = b;

        if(x                 ){fill_area(x-1,y  ,a,b);}
        if(y                 ){fill_area(x  ,y-1,a,b);}
        if(x < (Icon::size-1)){fill_area(x+1,y  ,a,b);}
        if(y < (Icon::size-1)){fill_area(x  ,y+1,a,b);}
    }
}


void
fill_area(int  x, int  y, uint16_t  pixel)
{
  auto  target = icon.data[y][x];

    if(target != pixel)
    {
      icon.data[y][x] = pixel;

        if(x                 ){fill_area(x-1,y  ,target,pixel);}
        if(y                 ){fill_area(x  ,y-1,target,pixel);}
        if(x < (Icon::size-1)){fill_area(x+1,y  ,target,pixel);}
        if(y < (Icon::size-1)){fill_area(x  ,y+1,target,pixel);}
    }
}




RadioBox*  radbox;

Counter*  r_cntr;
Counter*  g_cntr;
Counter*  b_cntr;

Button*  reverse_btn;
Button*  revolve_btn;
Button*  save_btn;
Button*  mirror_btn;


void
draw(Area&  area)
{
  area.fill(0);

    rep(y,Icon::size){
    rep(x,Icon::size){
      auto  d = icon.data[y][x];

        if(!(d&0x8000))
        {
          area.fill_rect(pixel_size*x,pixel_size*y,pixel_size,pixel_size,default_colors[d]);
        }

      else
        {
          area.fill_rect(pixel_size*x+1,pixel_size*y+1,pixel_size-2,pixel_size-2,default_colors[0]);
          area.fill_rect(pixel_size*x+2,pixel_size*y+2,pixel_size-4,pixel_size-4,default_colors[0x7FFF]);
        }
    }}


  area.draw_vline(pixel_size*(Icon::size/2)-1,0,(pixel_size*Icon::size),black_color);
  area.draw_vline(pixel_size*(Icon::size/2)  ,0,(pixel_size*Icon::size),white_color);
  area.draw_hline(0,pixel_size*(Icon::size/2)-1,(pixel_size*Icon::size),black_color);
  area.draw_hline(0,pixel_size*(Icon::size/2)  ,(pixel_size*Icon::size),white_color);
}


void
touch(Area&  area, const Mouse&  mouse)
{
  auto  pt = area.get_relative_point(mouse);

  int  x = pt.x/pixel_size;
  int  y = pt.y/pixel_size;

  auto  state = radbox->get_state();

    switch(state)
    {
      case(draw_point_flag):
        first_x = -1;
          if(mouse.test_left_pressing())
          {
            icon.data[y][x] = code;
          }

        else
          if(mouse.test_right_pressing())
          {
            icon.data[y][x] = 0x8000;
          }
        break;
      case(draw_line_flag):
          if(mouse.test_left_pressed())
          {
              if(first_x < 0)
              {
                first_x = x;
                first_y = y;
              }

            else
              {
                draw_line(first_x,first_y,x,y,code);

                first_x = -1;
              }
          }

        else
          if(mouse.test_right_pressed())
          {
              if(first_x < 0)
              {
                first_x = x;
                first_y = y;
              }

            else
              {
                draw_line(first_x,first_y,x,y,0x8000);

                first_x = -1;
              }
          }
        break;
      case(fill_area_flag):
        first_x = -1;

          if(mouse.test_left_pressed())
          {
            fill_area(x,y,code);
          }

        else
          if(mouse.test_right_pressed())
          {
            fill_area(x,y,0x8000);
          }
        break;
    }


  area.request_redraw();
}


void
update(Counter&  cntr, int  newval, int  oldval)
{
  int  r = r_cntr->get_cur();
  int  g = g_cntr->get_cur();
  int  b = b_cntr->get_cur();

  r = (r? ((r+1)*4)-1:0);
  g = (g? ((g+1)*4)-1:0);
  b = (b? ((b+1)*4)-1:0);

  code  = r<<10;
  code |= g<< 5;
  code |= b    ;
}


void
save(Button&  btn)
{
  auto  f = fopen(filename,"wb");

    if(f)
    {
        rep(y,Icon::size){
        rep(x,Icon::size){
          fprintf(f,"0x%04X,",icon.data[y][x]);
        }}


      fprintf(f,"\n");

      fclose(f);
    }
}


void
load()
{
  auto  f = fopen(filename,"rb");

    if(f)
    {
      int  n;

        rep(y,Icon::size){
        rep(x,Icon::size){
          fscanf(f," 0x%04X,",&n);

          icon.data[y][x] = n;
        }}


      fclose(f);
    }

  else
    {
      std::memset(&icon,0xFF,sizeof(icon));
    }
}


void
reverse(Button&  btn)
{
    if(btn->test_unpressed())
    {
        rep(y,Icon::size  ){
        rep(x,Icon::size/2){
          auto&  a = icon.data[y][           (  x)];
          auto&  b = icon.data[y][Icon::size-(1+x)];

          auto  t = a        ;
                    a = b    ;
                        b = t;
        }}


      drawing_area->request_redraw();
    }
}


void
revolve(Button&  btn)
{
    if(btn->test_unpressed())
    {
      tmp = icon;

        rep(y,Icon::size){
        rep(x,Icon::size){
          icon.data[y][           (  x)] =
           tmp.data[x][Icon::size-(1+y)];
        }}


      drawing_area->request_redraw();
    }
}


void
mirror(Button&  btn)
{
    if(btn->test_unpressed())
    {
        rep(y,Icon::size  ){
        rep(x,Icon::size/2){
          icon.data[y][Icon::size-(1+x)] =
          icon.data[y][           (  x)];
        }}


      drawing_area->request_redraw();
    }
}


}




int
main(int  argc, char**  argv)
{
  SDL_Init(SDL_INIT_VIDEO);

  load();

  auto  root = new Node;

  drawing_area = new Area(Icon::size*pixel_size,Icon::size*pixel_size);

  drawing_area->set_draw_callback(draw);
  drawing_area->set_callback(touch);


  radbox = new RadioBox;

  radbox->push(nullptr,u"点を打つ");
  radbox->push(nullptr,u"線を書く");
  radbox->push(nullptr,u"塗りつぶす");

  auto*  r_row = new TableRow;
  auto*  g_row = new TableRow;
  auto*  b_row = new TableRow;

  r_cntr = new Counter(0,l_max);
  g_cntr = new Counter(0,l_max);
  b_cntr = new Counter(0,l_max);

  r_cntr->set_callback(update);
  g_cntr->set_callback(update);
  b_cntr->set_callback(update);

  r_row->insert(new Label(u"R"));
  r_row->insert(r_cntr);
  g_row->insert(new Label(u"G"));
  g_row->insert(g_cntr);
  b_row->insert(new Label(u"B"));
  b_row->insert(b_cntr);


  auto  rgb_col = new TableColumn;
  
  rgb_col->insert(r_row);
  rgb_col->insert(g_row);
  rgb_col->insert(b_row);

  reverse_btn = new Button(u"反転");
  revolve_btn = new Button(u"回転");
  mirror_btn  = new Button(u"ミラー");
  save_btn = new Button(u"保存");

  reverse_btn->set_callback(reverse);
  revolve_btn->set_callback(revolve);
  save_btn->set_callback(save);
  mirror_btn->set_callback(mirror);

  root->insert(drawing_area,0,0);

  auto  col = new TableColumn;

  col->insert(radbox);
  col->insert(rgb_col);
  col->insert(reverse_btn);
  col->insert(revolve_btn);
  col->insert(mirror_btn);
  col->insert(save_btn);

  root->insert(col,Icon::size*pixel_size+8,0);

  sys::make_window("An ICON Editor - " __DATE__,64,64,root);


    while(sys::process_event())
    {
      sys::update();

      SDL_Delay(20);
    }


  SDL_Quit();

  return 0;
}




