#include"ireco.hpp"
#include<stdexcept>




namespace oat{




Ireco::
~Ireco()
{
    for(auto  ptr: children)
    {
      delete ptr;
    }
}


const Ireco&
Ireco::
operator[](int  i) const
{
  return *children[i];
}




bool
Ireco::
find(const char*  path, const Ireco*&  ireco) const
{
    for(auto  child: children)
    {
        if(child->name == path)
        {
          ireco = child;

          return true;
        }
    }


  return false;
}


bool
Ireco::
find(std::initializer_list<const char*>  path, int&  i) const
{
  const char*  s;

    if(find(path,s))
    {
      i = std::strtol(s,nullptr,10);

      return true;
    }


  return false;
}


bool
Ireco::
find(std::initializer_list<const char*>  path, const char*&  s) const
{
  const Ireco*  ireco;

    if(find(path,ireco))
    {
      s = ireco->value.data();

      return true;
    }


  return false;
}


bool
Ireco::
find(std::initializer_list<const char*>  path, const Ireco*&  ireco) const
{
  ireco = this;

    for(auto  dir: path)
    {
        if(!ireco->find(dir,ireco))
        {
          return false;
        }
    }


  return true;
}


bool
Ireco::
find(std::initializer_list<const char*>  path, const char*  name, std::vector<const Ireco*>&  ireco_buf) const
{
  const Ireco*  ireco;

    if(find(path,ireco))
    {
      int  n = 0;

        for(auto  child: ireco->children)
        {
            if(child->name == name)
            {
              ++n;

              ireco_buf.emplace_back(child);
            }
        }


      return n;
    }


  return false;
}




namespace{


bool
skip_spaces(const char*&  p)
{
    while(isspace(*p))
    {
      ++p;
    }


  return *p;
}


std::string  read_string(const char*&  p, const int  endc);


Ireco*
read_group(const char*&  p, const int  endc)
{
  auto  grp = new Ireco;

  std::string  s;

    while(skip_spaces(p))
    {
        if(*p == '[')
        {
          s = read_string(++p,']');
        }

      else
        if(*p == '(')
        {
          auto  vs = read_string(++p,')');

          auto  elm = new Ireco;

          elm->name.assign( std::move( s));
          elm->value.assign(std::move(vs));

          grp->children.emplace_back(elm);
        }

      else
        if(*p == '{')
        {
          auto  cogrp = read_group(++p,'}');

          cogrp->name.assign(std::move(s));

          grp->children.emplace_back(cogrp);
        }

      else
        if(*p == endc)
        {
          ++p;

          break;
        }
    }


  return grp;
}


std::string
read_string(const char*&  p, const int  endc)
{
  std::string  s;

    while(*p)
    {
        if(*p == endc)
        {
          ++p;

          break;
        }


        if(*p == '\\')
        {
          ++p;

            if(!*p)
            {
              break;
            }
        }


      s += *p++;
    }


  return std::move(s);
}


}


Ireco*
load_ireco(const char*  path)
{
  auto  bin = load_gzfile(path);

  bin.emplace_back(0);

  auto  p = (const char*)bin.data();

  return read_group(p,0);
}


}




