#include"cube.hpp"




void
CubePlane::
reset(CubePlaneID  _id, SheetPiece&  _first, PieceSideID  side_id)
{
  id = _id;

   first = _first;
  second = *_first[side_id].linked_piece;

  visible_flag = false;
}


void
CubePlane::
fill(Color  color)
{
   first.fill(color);
  second.fill(color);
}


void
CubePlane::
draw(void*  ptr, int  pitch) const
{
   first.draw(ptr,pitch);
  second.draw(ptr,pitch);
}




void
CubeElement::
make(SheetPiece&  base_piece)
{
  planes[PLN_top   ].reset(PLN_top   ,base_piece,SID_c);
  planes[PLN_bottom].reset(PLN_bottom,*base_piece.get_linked_piece({SID_b,SID_b}),SID_c);
  planes[PLN_front ].reset(PLN_front ,*base_piece.get_linked_piece({SID_b      }),SID_b);
  planes[PLN_back  ].reset(PLN_back  ,*base_piece.get_linked_piece({SID_c}),SID_b);
  planes[PLN_left  ].reset(PLN_left  ,base_piece,SID_b);
  planes[PLN_right ].reset(PLN_right ,*base_piece.get_linked_piece({SID_c,SID_b}),SID_b);
}




void
CubeElement::
set(uint8_t  l)
{
  planes[PLN_top].fill(map_rgb(l,l,l));
  planes[PLN_top].visible_flag = true;

int  n = l-20;

  planes[PLN_front].fill( map_rgb(n,n,n));
  planes[PLN_front].visible_flag = true;

n = l-40;
  planes[PLN_right].fill( map_rgb(n,n,n));
  planes[PLN_right].visible_flag = true;
}


void
CubeElement::
unset()
{
    for(auto&  pln: planes)
    {
      pln.visible_flag = false;
    }
}


void
CubeElement::
draw(void*  ptr, int  pitch) const
{
  auto  pln = &planes[PLN_front];

    if(pln->visible_flag)
    {
//      auto  front_el = linked_elements[PLN_front];

//        if(!front_el || !front_el->planes[PLN_top].visible_flag)
        {
          pln->draw(ptr,pitch);
        }
    }


  pln = &planes[PLN_top];

    if(pln->visible_flag)
    {
//      auto  front_el = linked_elements[PLN_front];

//        if(front_el && !front_el->planes[PLN_top].visible_flag)
        {
          pln->draw(ptr,pitch);
        }
    }


  pln = &planes[PLN_right];

    if(pln->visible_flag)
    {
//      auto  right_el = linked_elements[PLN_right];

//        if(!right_el || !right_el->planes[PLN_top].visible_flag)
        {
          pln->draw(ptr,pitch);
        }
    }
}




namespace{


void
make_links(Cube&  cb, const int  size, const int  x, const int  y, const int  z)
{
  auto&  el = cb.get(x,y,z);

  el.coord.x = x;
  el.coord.y = y;
  el.coord.z = z;

  el.linked_elements[PLN_top   ] = ((z           )? &cb.get(x  ,y  ,z-1):nullptr);
  el.linked_elements[PLN_bottom] = ((z < (size-1))? &cb.get(x  ,y  ,z+1):nullptr);
  el.linked_elements[PLN_back  ] = ((y           )? &cb.get(x  ,y-1,z  ):nullptr);
  el.linked_elements[PLN_front ] = ((y < (size-1))? &cb.get(x  ,y+1,z  ):nullptr);
  el.linked_elements[PLN_left  ] = ((x           )? &cb.get(x-1,y  ,z  ):nullptr);
  el.linked_elements[PLN_right ] = ((x < (size-1))? &cb.get(x+1,y  ,z  ):nullptr);
}


}




void
Cube::
make(Cube&  cb, Sheet&  sheet)
{
  const int  size = (sheet.width/2);

  cb.size = size;

  cb.elements.resize(size*size*size);

    rep(z,size){
    rep(y,size){
    rep(x,size){
      make_links(cb,size,x,y,z);
    }}}


  auto  z_piece = &sheet.get(size,0);

    rep(z,size)
    {
      auto  y_piece = z_piece                                           ;
                      z_piece = z_piece->get_linked_piece({SID_b,SID_b});

        rep(y,size)
        {
            if(!y_piece)
            {
              break;
            }


         auto  x_piece = y_piece                                           ;
                         y_piece = y_piece->get_linked_piece({SID_c,SID_b});

            rep(x,size)
            {
                if(!x_piece)
                {
                  break;
                }


              cb.get(x,y,z).make(*x_piece);

              x_piece = x_piece->get_linked_piece({SID_b,SID_c});
            }
        }
    }
}


void
Cube::
draw(Area&  area) const
{
  area.fill(0);

  auto  p = area.get_ptr(0,0);

    rrep(z,get_size())
    {
      int  n = 1;

        rep(y,get_size())
        {
            rep(x,n)
            {
              get(x,y-x,z).draw(p,sys::main_pitch);
            }


          ++n;
        }


    n -= 2;

        for(int  x = 1;  x < get_size();  ++x)
        {
            rep(y,n)
            {
              get(x+y,size-1-y,z).draw(p,sys::main_pitch);
            }


          --n;
        }
    }
}





