#include"image.hpp"




Image::Bucket::
Bucket():
r(0),
g(0),
b(0)
{
}




Image::
Image(int  w, int  h): pixels(w*h),
width(w),
height(h),
pitch(4*w)
{}


Image::
Image(Image&&  rhs):
pixels(std::move(rhs.pixels)),
width(rhs.width),
height(rhs.height),
pitch(rhs.pitch)
{}




Color&
Image::
get(int  x, int  y)
{
  return pixels[(width*y)+x];
}


const Color&
Image::
get(int  x, int  y) const
{
  return pixels[(width*y)+x];
}


Color
Image::
pick(int  x, int  y) const
{
  Bucket  buck;

  get_rgb(get(x,y),buck.r_base,buck.g_base,buck.b_base);

  buck.r = buck.r_base*2;
  buck.g = buck.g_base*2;
  buck.b = buck.b_base*2;


  int  n = 1;

    if(pick(x  ,y-1,buck,1)){++n;}
    if(pick(x  ,y-2,buck,2)){++n;}
    if(pick(x+1,y-1,buck,2)){++n;}
    if(pick(x+1,y  ,buck,1)){++n;}
    if(pick(x+2,y  ,buck,2)){++n;}
    if(pick(x+1,y+1,buck,2)){++n;}
    if(pick(x  ,y+1,buck,1)){++n;}
    if(pick(x  ,y+2,buck,2)){++n;}
    if(pick(x-1,y+1,buck,2)){++n;}
    if(pick(x-1,y  ,buck,1)){++n;}
    if(pick(x-2,y  ,buck,2)){++n;}
    if(pick(x-1,y-1,buck,2)){++n;}


  return map_rgb(buck.r/n,buck.g/n,buck.b/n);
}


bool
Image::
pick(int  x, int  y, Bucket&  buck, int  level) const
{
    if((x >=      0) &&
       (y >=      0) &&
       (x <   width) &&
       (y <  height))
    {
      uint8_t  sr;
      uint8_t  sg;
      uint8_t  sb;

      get_rgb(get(x,y),sr,sg,sb);

      buck.r += (buck.r_base+sr);
      buck.g += (buck.g_base+sg);
      buck.b += (buck.b_base+sb);

      return true;
    }


  return false;
}


void
Image::
save(const char*  path) const
{
  auto  bmp = SDL_CreateRGBSurfaceFrom((void*)pixels.data(),width,height,32,pitch,0x00FF0000,0x0000FF00,0x000000FF,0xFF000000);

  SDL_SaveBMP(bmp,path);

  SDL_FreeSurface(bmp);
}


Image
Image::
reduce() const
{
  Image  img(width/3,height/3);

    rep(y,img.height)
    {
      const int  yy = (y*3);

        if(yy < height)
        {
            rep(x,img.width)
            {
              const int  xx = (x*3);

                if(xx < width)
                {
                  img.get(x,y) = pick(xx,yy);
                }

              else
                {
                  break;
                }
            }
        }

      else
        {
          break;
        }
    }


  return std::move(img);
}




