#ifndef _ISOMT_PIECE_DRAW_HPP_
#define _ISOMT_PIECE_DRAW_HPP_


#include"piece.hpp"




template<typename  T>
struct
Surface
{
  uint8_t*  const ptr;

  const int  pitch;


  Surface(void*  _ptr, int  _pitch): ptr((uint8_t*)_ptr), pitch(_pitch){}

  T&  get(int  x, int  y)
  {
    auto  p = (T*)&ptr[pitch*y];

    return p[x];
  }

};


template<typename  T>
struct
ConstSurface
{
  const uint8_t*  const ptr;

  const int  pitch;


  ConstSurface(const void*  _ptr, int  _pitch): ptr((const uint8_t*)_ptr), pitch(_pitch){}

  const T&  get(int  x, int  y) const
  {
    auto  p = (const T*)&ptr[pitch*y];

    return p[x];
  }

};


template<typename  T>
void
put_surface(const ConstSurface<T>&  src, T*&  dst, int  x, int  y)
{
  *dst++ = src.get(x,y);
}


template<typename  T>
void
put_stream(const T*&  src, Surface<T>&  dst, int  x, int  y)
{
  dst.get(x,y) = *src++;
}


template<typename  T>
void
put_value(T  src, Surface<T>&  dst, int  x, int  y)
{
  dst.get(x,y) = src;
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_lt(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  x += side_length-1;

    rep(n,side_length)
    {
      put(src,dst,x,(y+n));
    }


  int  h = side_length;

    while(h)
    {
      --x;
      ++y;

      h -= 2;

        rep(n,h)
        {
          put(src,dst,x,(y+n));
        }


      --x;

        rep(n,h)
        {
          put(src,dst,x,(y+n));
        }
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_gt(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
    rep(n,side_length)
    {
      put(src,dst,x,(y+n));
    }


  int  h = side_length;

    while(h)
    {
      ++x;
      ++y;

      h -= 2;

        rep(n,h)
        {
          put(src,dst,x,(y+n));
        }


      ++x;

        rep(n,h)
        {
          put(src,dst,x,(y+n));
        }
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_lt_frame_a(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  x += side_length-2;

  int  h = (side_length/2);

    while(h--)
    {
      put(src,dst,x--,y  );
      put(src,dst,x--,y++);
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_lt_frame_b(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  x += side_length-2;
  y += side_length-2;

  int   h = (side_length/2);

    while(h--)
    {
      put(src,dst,x--,y  );
      put(src,dst,x--,y--);
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_lt_frame_c(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  x += side_length-1;

    rep(n,side_length)
    {
      put(src,dst,x,y++);
    }
}




template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_gt_frame_a(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  int  h = (side_length/2);

    while(h--)
    {
      put(src,dst,x++,y  );
      put(src,dst,x++,y++);
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_gt_frame_b(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
  y += side_length-2;

  int  h = (side_length/2);

    while(h--)
    {
      put(src,dst,x++,y  );
      put(src,dst,x++,y--);
    }
}


template<typename  PUTTER, typename  SRC, typename  DST>
void
draw_gt_frame_c(PUTTER&  put, SRC&  src, DST&  dst, int  x=0, int  y=0)
{
    rep(n,side_length)
    {
      put(src,dst,x,y++);
    }
}




#endif




