#include"sheet.hpp"
#include"piece_draw.hpp"




Area*     sheet_area;
Area*  blksheet_area;


std::vector<uint8_t>  Sheet::lt_masks;
std::vector<uint8_t>  Sheet::gt_masks;


namespace{


struct
Initializer
{
  Initializer();

};


Initializer::
Initializer()
{
  Sheet::lt_masks.resize(side_length*side_length);
  Sheet::gt_masks.resize(side_length*side_length);

    rep(y,side_length_half)
    {
        rep(x,side_length)
        {
          Sheet::lt_masks[side_length*(                 y)+x] = 0;
          Sheet::lt_masks[side_length*(side_length_half+y)+x] = 2;
          Sheet::gt_masks[side_length*(                 y)+x] = 0;
          Sheet::gt_masks[side_length*(side_length_half+y)+x] = 2;
        }
    }


  Surface<uint8_t>  lt_dst(Sheet::lt_masks.data(),side_length);
  Surface<uint8_t>  gt_dst(Sheet::gt_masks.data(),side_length);

  int  v = 1;

  draw_lt(put_value<uint8_t>,v,lt_dst);
  draw_gt(put_value<uint8_t>,v,gt_dst);
}


Initializer
init;


}


void
Sheet::
make_links(SheetPiece&  p, int  x, int  y)
{
  p.sides[SID_a].linked_piece = ((y             )? &get(x,y-1):nullptr);
  p.sides[SID_b].linked_piece = ((y < (height-1))? &get(x,y+1):nullptr);

    if(p.type == Piece::lt){p.sides[SID_c].linked_piece = ((x < (width -1))? &get(x+1,y):nullptr);}
  else                     {p.sides[SID_c].linked_piece = ((x             )? &get(x-1,y):nullptr);}
}


void
Sheet::
make(int  w, int  h)
{
  touched_piece = nullptr;

   width = w;
  height = h;

  sheets.resize(w*h);

    rep(x,w)
    {
      const int  xv = (side_length*x);

      bool  flag = (x&1);

        rep(y,h)
        {
          const int  yv = (side_length_half*y);

          auto&  p = get(x,y);

          p.point.x = xv;
          p.point.y = yv;

            for(auto&  side: p.sides)
            {
              side.color = map_rgb(127,127,127);
            }


          p.type = (flag? Piece::lt:Piece::gt);

          p.masks = (flag? lt_masks.data():gt_masks.data());

          make_links(p,x,y);

          flag = !flag;
        }
    }
}




SheetPiece&
Sheet::
get(int  x, int  y)
{
  return sheets[(width*y)+x];
}


const SheetPiece&
Sheet::
get(int  x, int  y) const
{
  return sheets[(width*y)+x];
}


SheetPiece*
Sheet::
get_piece(const SDL_Point&  pt)
{
  int  x = (pt.x/side_length     );
  int  y = (pt.y/side_length_half);

    if((x >= width ) ||
       (y >= height))
    {
      return nullptr;
    }


  int  x_fract = (pt.x%side_length     );
  int  y_fract = (pt.y%side_length_half);

  auto  p = &get(x,y);

  auto  mask_v = p->masks[(side_length*y_fract)+x_fract];

    if(mask_v == 0)
    {
        if(y)
        {
          p = &get(x,y-1);
        }
    }

  else
    if(mask_v == 2)
    {
        if(y < (height-1))
        {
          p = &get(x,y+1);
        }
    }


  return p;
}



void
Sheet::
draw(Area&  area, bool  framing)
{
  area.fill(0);

  auto  p = area.get_ptr(0,0);

    rep(y,height){
    rep(x,width ){
      get(x,y).draw(p,sys::main_pitch,framing);
    }}
}



void
Sheet::
touch(Area&  area, const SDL_Point&  pt, const SheetPiece*  piece)
{
  auto  p = get_piece(pt);

    if(p)
    {
      touched_piece = p;

        if(piece)
        {
          p->copy_from(piece->pixels);

          area.request_redraw();
        }
    }
}




void
Sheet::
get_image_size(int&  w, int&  h) const
{
  w = (side_length*width);
  h = ((side_length/2)*(height+1));
}


void
Sheet::
write_to(Image&  img) const
{
    rep(y,height){
    rep(x,width){
      get(x,y).draw(img.pixels.data(),img.pitch);
    }}
}



void
Sheet::
read_from(const Image&  img)
{
    rep(y,height){
    rep(x,width){
      get(x,y).copy_from(img.pixels.data(),img.pitch);
    }}
}





