#include"sheet.hpp"
#include"piece_draw.hpp"




SheetPiece::Side&
SheetPiece::
operator[](PieceSideID  id)
{
  return sides[id];
}


SheetPiece*
SheetPiece::
get_linked_piece(std::initializer_list<PieceSideID>  ls)
{
  auto  p = this;

    for(auto  id: ls)
    {
      p = p->sides[id].linked_piece;

       if(!p)
       {
         break;
       }
    }


  return p;
}


void
SheetPiece::
reset(Color*  _pixels)
{
    for(auto&  side: sides)
    {
      side.color = get_default_color(15);
    }


  copy_from(_pixels);
}




void
SheetPiece::
fill(Color  color)
{
  auto  p = pixels;

  int  n = (side_length*side_length);

    while(n--)
    {
      *p++ = color;
    }
}


void
SheetPiece::
copy_from(const Color*  src)
{
    if(src)
    {
      std::memcpy(pixels,src,(sizeof(*pixels)*side_length*side_length));
    }
}


void
SheetPiece::
copy_from(const void*  ptr, int  pitch)
{
  ConstSurface<uint32_t>  src(ptr,pitch);

  auto  p = pixels;

    switch(type)
    {
      case(lt):
        draw_lt(put_surface<uint32_t>,src,p,point.x,point.y);
        break;
      case(gt):
        draw_gt(put_surface<uint32_t>,src,p,point.x,point.y);
        break;
    }
}


void
SheetPiece::
draw(void*  ptr, int  pitch, bool  framing) const
{
  Surface<Color>  dst(ptr,pitch);

  const Color*  p = pixels;

    switch(type)
    {
      case(lt):
        draw_lt(put_stream<uint32_t>,p,dst,point.x,point.y);

          if(framing)
          {
            draw_lt_frame_a(put_value<uint32_t>,sides[SID_a].color,dst,point.x,point.y);
            draw_lt_frame_b(put_value<uint32_t>,sides[SID_b].color,dst,point.x,point.y);
            draw_lt_frame_c(put_value<uint32_t>,sides[SID_c].color,dst,point.x,point.y);
          }
        break;
      case(gt):
        draw_gt(put_stream<uint32_t>,p,dst,point.x,point.y);

          if(framing)
          {
            draw_gt_frame_a(put_value<uint32_t>,sides[SID_a].color,dst,point.x,point.y);
            draw_gt_frame_b(put_value<uint32_t>,sides[SID_b].color,dst,point.x,point.y);
            draw_gt_frame_c(put_value<uint32_t>,sides[SID_c].color,dst,point.x,point.y);
          }
        break;
    }
}





