#include"menu.hpp"
#include"font.hpp"
#include"color.hpp"




namespace oat{


namespace{


SDL_Window*      window;
SDL_Renderer*  renderer;
SDL_Texture*    texture;

std::vector<MenuItem>  items;


int
pos;


int  w;
int  h;


}


void
make_menu(std::initializer_list<MenuItem>  ls, int  x, int  y)
{
  pos = 0;

  w = 0;
  h = font::h*ls.size();

    for(auto&  item: ls)
    {
      items.emplace_back(item);

      w = std::max(w,(int)font::get_width(item.name));
    }


  window = SDL_CreateWindow("menu",x,y,w,h,SDL_WINDOW_BORDERLESS|SDL_WINDOW_INPUT_GRABBED);

  renderer = SDL_CreateRenderer(window,-1,0);

  texture = SDL_CreateTexture(renderer,colorformat,SDL_TEXTUREACCESS_STREAMING,w,h);
}


void
clean_menu()
{
  items.clear();

  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);
}


bool
test_menu()
{
  return items.size();
}


void
move_menu_cursor(int  cursor_y)
{
  pos = cursor_y/font::h;
}


void
execute_menu_callback()
{
  items[pos].callback(pos);
}


void
update_menu()
{
  void*  ptr;

  int  pitch;

  SDL_LockTexture(texture,nullptr,&ptr,&pitch);

  auto  pixels = (uint8_t*)ptr;

  int  y = 0;

    for(auto&  item: items)
    {
      bool  focused = (y == pos);

        rep(yy,font::h)
        {
          auto  dst = (uint32_t*)&pixels[pitch*((font::h*y)+yy)];

            rep(x,w)
            {
              *dst++ = focused? black_color:light_gray_color;
            }
        }


      font::draw_unicode(item.name,pixels,pitch,0,(font::h*y),focused? white_color: black_color);

      ++y;
    }


  SDL_UnlockTexture(texture);


  SDL_RenderClear(renderer);

  SDL_RenderCopy(renderer,texture,nullptr,nullptr);

  SDL_RenderPresent(renderer);
}


}




