#include"mouse.hpp"




namespace oat{


enum{
  l_pressed   = 0x01,
  l_unpressed = 0x02,
  r_pressed   = 0x04,
  r_unpressed = 0x08,
};


Mouse::
Mouse()
{
  clear();
}




bool    Mouse::test_left_pressing() const{return left_pressing;}
bool    Mouse::test_left_pressed() const{return(tmp_flags&l_pressed);}
bool  Mouse::test_left_unpressed() const{return(tmp_flags&l_unpressed);}


void
Mouse::
press_left()
{
    if(!left_pressing)
    {
      left_pressing = true;

      tmp_flags |= l_pressed;
    }
}


void
Mouse::
unpress_left()
{
    if(left_pressing)
    {
      left_pressing = false;

      tmp_flags |= l_unpressed;
    }
}




bool    Mouse::test_right_pressing() const{return right_pressing;}
bool    Mouse::test_right_pressed() const{return(tmp_flags&r_pressed);}
bool  Mouse::test_right_unpressed() const{return(tmp_flags&r_unpressed);}


void
Mouse::press_right()
{
    if(!right_pressing)
    {
      right_pressing = true;

      tmp_flags |= r_pressed;
    }
}


void
Mouse::
unpress_right()
{
    if(right_pressing)
    {
      right_pressing = false;

      tmp_flags |= r_unpressed;
    }
}




void
Mouse::
clean()
{
  tmp_flags = 0;
}




int  Mouse::get_abs_x() const{return abs_x;}
int  Mouse::get_abs_y() const{return abs_y;}


bool  Mouse::test_motion() const{return(motion_flag);}


void
Mouse::
button(const SDL_MouseButtonEvent&  evt)
{
  abs_x = evt.x;
  abs_y = evt.y;

  motion_flag = false;

    switch(evt.state)
    {
      case(SDL_PRESSED):
          switch(evt.button)
          {
            case(SDL_BUTTON_LEFT ): press_left();break;
            case(SDL_BUTTON_RIGHT): press_right();break;
          }
        break;
      case(SDL_RELEASED):
          switch(evt.button)
          {
            case(SDL_BUTTON_LEFT ): unpress_left();break;
            case(SDL_BUTTON_RIGHT): unpress_right();break;
          }
        break;
    }
}


void
Mouse::
motion(const SDL_MouseMotionEvent&  evt)
{
  abs_x = evt.x;
  abs_y = evt.y;

  motion_flag = true;

    if(evt.state&SDL_BUTTON_LMASK){  press_left();}
  else                            {unpress_left();}


    if(evt.state&SDL_BUTTON_RMASK){  press_right();}
  else                            {unpress_right();}
}


void
Mouse::
clear()
{
  std::memset(this,0,sizeof(*this));
}


void
Mouse::
print() const
{
  printf("[oat::mouse print] %4d:%4d\n",abs_x,abs_y);
}




}




