#include"painter_private.hpp"




namespace painter{




namespace{


void
change_mode(CheckBox&  chkbox, uint32_t  newval, uint32_t  opldval)
{
  frame.reset();

  drawing_paper->merge( string_buffer,Paper::UDO_string);
  drawing_paper->unputs(stroke_buffer);

  using_frame = (newval == PAI_set_frame);

  widgets.drawing_area->request_redraw();
}






void
undo(Button&  btn)
{
    if(btn->test_pressing())
    {
      auto&  buf = string_buffer;

        if(buf.size())
        {
          drawing_paper->unput(buf);
        }

      else
        {
          drawing_paper->undo();
        }


      widgets.drawing_area->request_redraw();

      update();
    }
}


void
copy(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  rect = make_rect(*frame);

      copy_paper = drawing_paper->make_copy(rect);
    }
}


void
grid(Button&  btn)
{
  using_grid = btn->test_pressing();

  widgets.drawing_area->request_redraw();
}




}




void
Widgets::
initialize()
{
  static bool  done;

    if(done)
    {
      return;
    }


  brush_label = new Label(u"Brush(X: XXXX +XXXX, Y: XXXX +XXXX)");
  frame_label = new Label(u"Frame(X: XXXX, Y: XXXX, W: XXXX, H: XXXX)");

  radiobox = new RadioBox;

  radiobox->push(nullptr,u"点を打つ");
  radiobox->push(nullptr,u"線を引く");
  radiobox->push(nullptr,u"円を描く");
  radiobox->push(nullptr,u"矩形を描く");
  radiobox->push(nullptr,u"矩形を塗る");
  radiobox->push(nullptr,u"領域を塗る");
  radiobox->push(nullptr,u"コピーを置く");
  radiobox->push(nullptr,u"選択");


  radiobox->set_callback(change_mode);


  copy_button = new RotorButton(u"コピー");

  copy_button->set_callback(copy);


  undo_button = new Button(u"アンドゥ");

  undo_button->set_callback(undo,true);


  grid_button = new Button(u"グリッド");

  grid_button->set_callback(grid);
  grid_button->set_toggle();


  reverse_widget  = create_reverse_widget();
  revolve_widget  = create_revolve_widget();
  mirror_widget   = create_mirror_widget();
  shift_ud_widget = create_udshift_widget();
  shift_lr_widget = create_lrshift_widget();



  drawing_area = new Area(whole_size.x*whole_pixel_size,
                          whole_size.y*whole_pixel_size,8);

  drawing_area->set_flag(WID_ST_hide_cursor_flag);

  drawing_area->set_draw_callback(draw_whole);
  drawing_area->set_callback(react_whole);


  part_drawing_area = new Area(part_rect.w*part_pixel_size,
                               part_rect.h*part_pixel_size,8);

  part_drawing_area->set_flag(WID_ST_hide_cursor_flag);

  part_drawing_area->set_draw_callback(draw_part);
  part_drawing_area->set_callback(react_part);

  done = true;
}


void
Widgets::
update_labels()
{
  auto  rect = make_rect(*frame);

  brush_label->printf("Brush(X: %4d +%4d, Y: %4d +%4d)",part_rect.x,x_offset,
                                                        part_rect.y,y_offset);

  frame_label->printf("Frame(X: %4d, Y: %4d, W: %4d, H: %4d)",rect.x,rect.y,rect.w,rect.h);
}




}




