#include"painter_private.hpp"




namespace painter{




void
draw_grid(Area&  area, const SDL_Rect&  rect, int  pixel_size)
{
    rep(y,rect.h)
    {
      const int  n = (rect.y+y);

      const int  yy = (pixel_size*y);

           if(!(n&31)){area.draw_hline(0,yy,area.get_parameter().width,get_default_color(15));}
      else if(!(n& 7)){area.draw_hline(0,yy,area.get_parameter().width,get_default_color( 3));}
    }


    rep(x,rect.w)
    {
      const int  n = (rect.x+x);

      const int  xx = (pixel_size*x);

           if(!(n&31)){area.draw_vline(xx,0,area.get_parameter().height,get_default_color(15));}
      else if(!(n& 7)){area.draw_vline(xx,0,area.get_parameter().height,get_default_color( 3));}
    }
}


void
draw_whole(Area&  area)
{
  area.fill(palette->get_color(0));

    for(auto  pp: showing_papers)
    {
      pp->draw(area,0,
                    0,nullptr,whole_pixel_size,*palette);
    }


    if(state == 0)
    {
        if(using_grid)
        {
          SDL_Rect  rect = {0,0,whole_size.x,whole_size.y};

          draw_grid(area,rect,whole_pixel_size);
        }


        if(using_frame)
        {
          auto  frame_rect = make_rect(*frame);

          int  x = frame_rect.x*whole_pixel_size;
          int  y = frame_rect.y*whole_pixel_size;
          int  w = frame_rect.w*whole_pixel_size;
          int  h = frame_rect.h*whole_pixel_size;

          area.draw_rect_safety(x,y,w,h,white_color,true);
        }


      int  x = (whole_pixel_size*x_offset);
      int  y = (whole_pixel_size*y_offset);

        if(whole_pixel_size == 1)
        {
          area.draw_rect(x ,y  ,2,2,black_color);
          area.draw_dot(x+1,y+1,    white_color);
        }

      else
        if(whole_pixel_size == 2)
        {
          area.draw_rect(x  ,y  ,3,3,black_color);
          area.draw_rect(x+1,y+1,1,1,white_color);
        }

      else
        {
          area.draw_rect(x  ,y  ,whole_pixel_size  ,whole_pixel_size  ,black_color);
          area.draw_rect(x+1,y+1,whole_pixel_size-2,whole_pixel_size-2,white_color);
        }
    }


    if(base_mover_using)
    {
      const int  x = whole_pixel_size*part_rect.x;
      const int  y = whole_pixel_size*part_rect.y;
      const int  w = whole_pixel_size*part_rect.w;
      const int  h = whole_pixel_size*part_rect.h;

      area.draw_rect(x  ,y  ,w  ,h  ,black_color);
      area.draw_rect(x+1,y+1,w-2,h-2,white_color);
    }
}




void
draw_part(Area&  area)
{
  area.fill(palette->get_color(0));

    for(auto  pp: showing_papers)
    {
      pp->draw(area,0,
                    0,&part_rect,part_pixel_size,*palette);
    }


    if(state == 1)
    {
        if(using_grid)
        {
          draw_grid(area,part_rect,part_pixel_size);
        }


        if(using_frame)
        {
          auto  frame_rect = make_rect(*frame);

          int  x = frame_rect.x*part_pixel_size-(part_rect.x*part_pixel_size);
          int  y = frame_rect.y*part_pixel_size-(part_rect.y*part_pixel_size);
          int  w = frame_rect.w*part_pixel_size;
          int  h = frame_rect.h*part_pixel_size;

          area.draw_rect_safety(x,y,w,h,white_color,true);
        }


      int  x = (part_pixel_size*x_offset);
      int  y = (part_pixel_size*y_offset);

        if(part_pixel_size == 1)
        {
          area.draw_rect(x ,y  ,2,2,black_color);
          area.draw_dot(x+1,y+1,    white_color);
        }

      else
        if(part_pixel_size == 2)
        {
          area.draw_rect(x  ,y  ,3,3,black_color);
          area.draw_rect(x+1,y+1,1,1,white_color);
        }

      else
        {
          area.draw_rect(x  ,y  ,part_pixel_size  ,part_pixel_size  ,black_color);
          area.draw_rect(x+1,y+1,part_pixel_size-2,part_pixel_size-2,white_color);
        }
    }
}




}





