#include"painter_private.hpp"




namespace painter{




namespace{




void
react_2p(const Mouse&  mouse, int  x, int  y, Paper::TwoPointFunction  fn)
{
    if(mouse.test_left_pressing())
    {
      frame.set(x,y);

      drawing_paper->merge( string_buffer,Paper::UDO_string);
      drawing_paper->unputs(stroke_buffer);

      (drawing_paper->*fn)(*frame,palette->make_pixel(),&stroke_buffer);
    }

  else
    {
      frame.reset();

        if(mouse.test_left_unpressed())
        {
          drawing_paper->merge(stroke_buffer,Paper::UDO_stroke);
        }

      else
        {
          drawing_paper->unputs(stroke_buffer);
        }
    }
}


void
react_sub(const Mouse&  mouse, int  x, int  y)
{
    if(mouse.test_right_pressing())
    {
        if(frame)
        {
          frame.reset();
        }

      else
        if(!base_mover_using)
        {
          auto  pix = drawing_paper->get_pixel(x,y);

          palette->find(pix);
        }


      return;
    }



    switch(widgets.radiobox->get_state())
    {
      case(PAI_draw_point):
          if(mouse.test_left_pressing())
          {
            drawing_paper->put(x,y,palette->make_pixel(),&string_buffer);
          }
        break;
      case(PAI_draw_line  ): react_2p(mouse,x,y,&Paper::draw_line  );break;
      case(PAI_draw_circle): react_2p(mouse,x,y,&Paper::draw_circle);break;
      case(PAI_draw_rect  ): react_2p(mouse,x,y,&Paper::draw_rect  );break;
      case(PAI_fill_rect  ): react_2p(mouse,x,y,&Paper::fill_rect  );break;

      case(PAI_set_frame):
          if(mouse.test_left_pressing())
          {
            frame.set(x,y);
          }
        break;
      case(PAI_fill_area):
          if(mouse.test_left_pressed())
          {
            drawing_paper->fill_area(x,y,palette->make_pixel(),&stroke_buffer);

            drawing_paper->merge(stroke_buffer,Paper::UDO_stroke);
          }
        break;
      case(PAI_put_copy):
          if(mouse.test_left_pressing())
          {
            drawing_paper->merge(stroke_buffer,Paper::UDO_stroke);
          }

        else
          {
            drawing_paper->unputs(stroke_buffer);

            drawing_paper->put(x,y,copy_paper,&stroke_buffer);
          }
        break;
      default:;
    }
}


void
set_offset(int&  x, int&  y, int  pixel_size)
{
  x /= pixel_size;
  y /= pixel_size;

  x_offset = x;
  y_offset = y;
}




void
move_part_rect(int  x, int  y)
{
  part_rect.x = x-(part_rect.w/2);
  part_rect.y = y-(part_rect.h/2);

    if(part_rect.x < 0)
    {
      part_rect.x = 0;
    }

  else
    if((part_rect.x+part_rect.w) >= drawing_paper->get_w())
    {
      part_rect.x = drawing_paper->get_w()-part_rect.w;
    }


    if(part_rect.y < 0)
    {
      part_rect.y = 0;
    }

  else
    if((part_rect.y+part_rect.h) >= drawing_paper->get_h())
    {
      part_rect.y = drawing_paper->get_h()-part_rect.h;
    }
}


}




void
react_whole(Area&  area, const Mouse&  mouse)
{
  auto  pt = area.get_relative_point(mouse);

  state = 0;

  int  x = pt.x;
  int  y = pt.y;

  set_offset(x,y,whole_pixel_size);

    if(mouse.test_right_pressing() && base_mover_using)
    {
      move_part_rect(x,y);
    }

  else
    {
      react_sub(mouse,x,y);
    }


  widgets.update_labels();

  area.request_redraw();

    if(widgets.part_drawing_area)
    {
      widgets.part_drawing_area->request_redraw();
    }


    for(auto  ptr: relative_widgets)
    {
      ptr->request_redraw();
    }
}




void
react_part(Area&  area, const Mouse&  mouse)
{
  auto  pt = area.get_relative_point(mouse);

  state = 1;

  int  x = pt.x;
  int  y = pt.y;

  set_offset(x,y,part_pixel_size);

  react_sub(mouse,part_rect.x+x,part_rect.y+y);


  widgets.update_labels();

    if(widgets.drawing_area)
    {
      widgets.drawing_area->request_redraw();
    }


  area.request_redraw();

    for(auto  ptr: relative_widgets)
    {
      ptr->request_redraw();
    }
}




}




