#include"palette8.hpp"
#include"colorchooser.hpp"




using namespace oat;


namespace painter{




Palette8::
Palette8()
{
  x_pos = 0;
  y_pos = 0;
}




int  Palette8::get_index() const{return((y_pos<<3)|x_pos);}
int  Palette8::get_x_pos() const{return x_pos;}
int  Palette8::get_y_pos() const{return y_pos;}

void  Palette8::set_x_pos(int  x){x_pos = x;}
void  Palette8::set_y_pos(int  y){y_pos = y;}


Color  Palette8::get_base_color(      ) const{return colors[y_pos][0];}
Color  Palette8::get_base_color(int  y) const{return colors[y    ][0];}

Color  Palette8::get_color(              ) const{return colors[y_pos][x_pos];}
Color  Palette8::get_color(Paper::Pixel i) const{return colors[i>>3][i&7];}
Color  Palette8::get_color(int  x, int  y) const{return colors[y][x];}


const Color*  Palette8::get_pointer() const{return &colors[0][0];}


void
Palette8::
set_color(int  r, int  g, int  b)
{
  set_color(y_pos,r,g,b);
}


namespace{


int
fixv(int  v)
{
       if(v <   0){return   0;}
  else if(v > 255){return 255;}

  return v;
}


}


void
Palette8::
set_color(int  y, int  r, int  g, int  b)
{
  colors[y][0] = map_rgb(r,g,b);

  const int  v[] = {-191,-127,-63,0,63,127,191};

    rep(x,w-1)
    {
      int  tr = fixv(r+v[x]);
      int  tg = fixv(g+v[x]);
      int  tb = fixv(b+v[x]);

      colors[y][1+x] = map_rgb(tr,tg,tb);
    }
}


Paper::Pixel
Palette8::
make_pixel() const
{
  return get_index();
}


void
Palette8::
find(Paper::Pixel  pix)
{
  set_x_pos(pix&7);
  set_y_pos(pix>>3);

  palette8.widgets.area->request_redraw();

  colorchooser.widgets.update();
}




void
Palette8::
read(const oat::Ireco&  grp)
{
  int  i = 0;

  const int  t = std::stoi(grp[i++].value);

    rep(y,t)
    {
      int  r;
      int  g;
      int  b;

        if(y < h)
        {
          auto  p = grp[i++].value.data();

          sscanf(p,"%02X%02X%02X",&r,&g,&b);

          set_color(y,r,g,b);
        }
    }
}


void
Palette8::
write(std::string&  buf) const
{
  buf += "[palette8]{";


  buf += "[h](";
  buf += std::to_string(h);
  buf += ")\n";

  char  cbuf[16];

    rep(y,h)
    {
      uint8_t  r;
      uint8_t  g;
      uint8_t  b;

      get_rgb(get_color(0,y),r,g,b);

      snprintf(cbuf,sizeof(cbuf),"%02X%02X%02X",r,g,b);

      buf += "[rgb](";
      buf += cbuf;
      buf += ")\n";
    }


  buf += "}\n";
}




Palette8
palette8;


}




