#include"paper.hpp"
#include"paper_dot.hpp"
#include"../pngbuffer.hpp"




using namespace oat;




Paper::
Paper(int  _w, int  _h):
pixels(_w*_h),
width(_w),
height(_h),
undo_buffer(nullptr)
{
}


Paper::
Paper(const Paper&  rhs)
{
  *this = rhs;
}


Paper::
Paper(Paper&&  rhs)
{
  swap(rhs);
}




void
Paper::
operator=(const Paper&  rhs)
{
  width  = rhs.width ;
  height = rhs.height;

  pixels = rhs.pixels;

    if(undo_buffer)
    {
        if(rhs.undo_buffer)
        {
          *undo_buffer = *rhs.undo_buffer;
        }

      else
        {
          undo_buffer->clear();
        }
    }

  else
    if(rhs.undo_buffer)
    {
      undo_buffer = new UndoBuffer(*rhs.undo_buffer);
    }
}


void
Paper::
operator=(Paper&&  rhs)
{
  swap(rhs);
}


void
Paper::
swap(Paper&  rhs)
{
  std::swap(pixels,rhs.pixels);
  std::swap(width ,rhs.width );
  std::swap(height,rhs.height);
  std::swap(undo_buffer,rhs.undo_buffer);
}


int  Paper::get_w() const{return  width;}
int  Paper::get_h() const{return height;}



      Paper::Pixel&  Paper::get_pixel(int  x, int  y)      {return pixels[width*y+x];}
const Paper::Pixel&  Paper::get_pixel(int  x, int  y) const{return pixels[width*y+x];}




void
Paper::
set_undo_buffer(UndoBuffer*  buf)
{
  undo_buffer = buf;
}


void
Paper::
resize(int  _w, int  _h)
{
  width  = _w;
  height = _h;

  pixels.resize(_w*_h);
}


void
Paper::
put(int  x, int  y, Pixel  pixel, DotString*  buf)
{
  auto&  dst = get_pixel(x,y);

    if(dst != pixel)
    {
        if(buf)
        {
          buf->emplace_back(Dot(x,y,dst).code);
        }


      dst = pixel;
    }
}


void
Paper::
put(int  x, int  y, const Paper&  src, DotString*  buf)
{
  const int  w = (((x+src.get_w()) < get_w())? src.get_w():(get_w()-x));
  const int  h = (((y+src.get_h()) < get_h())? src.get_h():(get_h()-y));

    rep(yy,h){
    rep(xx,w){
      put(x+xx,y+yy,src.get_pixel(xx,yy),buf);
    }}
}


void
Paper::
pile(int  x, int  y, const Paper&  src, DotString*  buf)
{
  const int  w = (((x+src.get_w()) < get_w())? src.get_w():(get_w()-x));
  const int  h = (((y+src.get_h()) < get_h())? src.get_h():(get_h()-y));

    rep(yy,h){
    rep(xx,w){
      auto  pix = src.get_pixel(xx,yy);

        if(pix)
        {
          put(x+xx,y+yy,pix,buf);
        }
    }}
}


void
Paper::
unput(DotString&  buf)
{
    if(buf.size())
    {
      int  x;
      int  y;
      int  pixel;

      Dot(buf.back()).get(x,y,pixel);

      buf.pop_back();

      get_pixel(x,y) = pixel;
    }
}


void
Paper::
unputs(DotString&  buf)
{
    for(auto  v: buf)
    {
      int  x;
      int  y;
      int  pixel;

      Dot(v).get(x,y,pixel);

      get_pixel(x,y) = pixel;
    }


  buf.clear();
}


void
Paper::
merge(DotString&  buf, UndoType  type)
{
    if(!undo_buffer || buf.empty())
    {
      goto QUIT;
    }


  undo_buffer->emplace_back(std::make_pair(type,std::move(buf)));

    if(undo_buffer->size() > 1200)
    {
      undo_buffer->pop_front();
    }


QUIT:
  buf.clear();
}


void
Paper::
undo()
{
    if(!undo_buffer)
    {
      return;
    }


  auto&  buf = *undo_buffer;

    if(buf.empty())
    {
      return;
    }


  auto&  dat = buf.back();

  auto&  dots = dat.second;

    switch(dat.first)
    {
      case(UDO_string):
        unput(dots);
        break;
      case(UDO_stroke):
        unputs(dots);
        break;
    }


    if(dots.empty())
    {
      buf.pop_back();
    }
}




Paper
Paper::
make_copy(const SDL_Rect&  rect, CopyType  type) const
{
  int  w = rect.w;
  int  h = rect.h;

    switch(type)
    {
      case(CPY_revolve_left ):
      case(CPY_revolve_right):
        w = std::min(w,h);
        h = std::min(w,h);
        break;
      default:;
    };


  Paper  chk(w,h);

    switch(type)
    {
      case(CPY_nochange):
          rep(y,h){
          rep(x,w){
            chk.put(x,y,get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_mirror_upper):
          rep(y,h/2){
          rep(x,w  ){
            chk.put(x,(  (  y)),get_pixel(rect.x+x,rect.y+y));
            chk.put(x,(h-(1+y)),get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_mirror_left):
          rep(y,h  ){
          rep(x,w/2){
            chk.put((  (  x)),y,get_pixel(rect.x+x,rect.y+y));
            chk.put((w-(1+x)),y,get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_revolve_left):
          rep(y,h){
          rep(x,w){
            chk.put(y,h-(1+x),get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_revolve_right):
          rep(y,h){
          rep(x,w){
            chk.put(w-(1+y),x,get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_reverse_horizontal):
          rep(y,h){
          rep(x,w){
            chk.put(x,(h-(1+y)),get_pixel(rect.x+x,rect.y+y));
          }}
        break;
      case(CPY_reverse_vertical):
          rep(y,h){
          rep(x,w){
            chk.put((w-(1+x)),y,get_pixel(rect.x+x,rect.y+y));
          }}
        break;
    }


  return std::move(chk);
}




void
Paper::
save_as_png(const char*  path, int  color_number, const Color*  src_colors) const
{
  PNGBuffer   buf;

  buf.make_indexed_color(get_w(),get_h(),color_number,src_colors);

    rep(y,get_h()){
    rep(x,get_w()){
      *buf.get_index(x,y) = get_pixel(x,y);
    }}


  buf.save(path);
}




void
Paper::
draw(Area&  area, int  x_base, int  y_base, const SDL_Rect*  rect, int  pixel_size, const Palette&  pal) const
{
  SDL_Rect  tmp_rect;

    if(!rect)
    {
      tmp_rect.x = 0;
      tmp_rect.y = 0;
      tmp_rect.w = get_w();
      tmp_rect.h = get_h();

      rect = &tmp_rect;
    }


    rep(y,rect->h){
    rep(x,rect->w){
      const int  xx = (rect->x+x);
      const int  yy = (rect->y+y);

      auto  pixel = get_pixel(xx,yy);

        if(pixel)
        {
          area.fill_rect(x_base+(pixel_size*x),
                         y_base+(pixel_size*y),pixel_size,
                                               pixel_size,pal.get_color(pixel));
        }
    }}
}




void
Paper::
read(const oat::Ireco&  grp)
{
  int  w;
  int  h;

  const Ireco*  pixels_ireco;

    if(grp.find({"width"},w) &&
       grp.find({"height"},h) &&
       grp.find({"pixels"},pixels_ireco))
    {
      const Ireco*  base64_ireco;

        if(grp.find({"base64"},base64_ireco))
        {
          pixels = decode_base64(pixels_ireco->value);
        }

      else
        {
          auto  p = pixels_ireco->value.data();

          int  n;

            rep(y,h){
            rep(x,w){
              sscanf(p,"%02X",&n);

              get_pixel(x,y) = n;

              p += 2;
            }}
        }
    }
}


void
Paper::
write(std::string&  buf) const
{
  buf += "[paper]{";

  buf += "[base64](1)\n";

  buf += "[width](";
  buf += std::to_string(get_w());
  buf += ")\n";

  buf += "[height](";
  buf += std::to_string(get_h());
  buf += ")\n";

  buf += "[pixels](";

  auto  s = encode_base64(pixels.data(),pixels.size());

  buf += s;

  buf += ")\n";
  buf += "}\n";
}




