#ifndef _PAPER_
#define _PAPER_


#include<cstdint>
#include<vector>
#include<deque>
#include<memory>
#include<utility>
#include"../oat.hpp"




class
Paper
{
public:
  struct Dot;
  class Frame;
  class Palette;

  using Pixel = uint8_t;

  using DotString = std::vector<uint32_t>;

  using TwoPointFunction = void  (Paper::*)(const oat::TwoPoints&  points, Pixel  pixel, DotString*  buf);
  using    FrameFunction = void  (Paper::*)(const oat::TwoPoints&  points              , DotString*  buf);

  enum UndoType
  {
    UDO_string,
    UDO_stroke,
  };


  using UndoData = std::pair<UndoType,DotString>;
  using UndoBuffer = std::deque<UndoData>;


private:
  std::vector<Pixel>  pixels;

  int   width;
  int  height;

  UndoBuffer*  undo_buffer;

public:
  enum CopyType
  {
    CPY_nochange,
    CPY_mirror_left,
    CPY_mirror_upper,
    CPY_revolve_left,
    CPY_revolve_right,
    CPY_reverse_vertical,
    CPY_reverse_horizontal,

  };



  Paper(int  _w=0, int  _h=0);
  Paper(const Paper&   rhs);
  Paper(      Paper&&  rhs);


  void  operator=(const Paper&   rhs);
  void  operator=(      Paper&&  rhs);

  void  swap(Paper&  rhs);

  int  get_w() const;
  int  get_h() const;


  void  set_undo_buffer(UndoBuffer*  buf);

  void  resize(int  _w, int  _h);

  void  put(int  x, int  y,       Pixel   pixel, DotString*  buf=nullptr);
  void  put(int  x, int  y, const Paper&  Paper, DotString*  buf=nullptr);
  void  pile(int  x, int  y, const Paper&  Paper, DotString*  buf=nullptr);

  void  unput( DotString&  buf);
  void  unputs(DotString&  buf);

  void  merge(DotString&  buf, UndoType  type);

  void  undo();

        Pixel&  get_pixel(int  x, int  y)      ;
  const Pixel&  get_pixel(int  x, int  y) const;

  Paper  make_copy(const SDL_Rect&  rect, CopyType  type=CPY_nochange) const;


  void    draw_line(const oat::TwoPoints&  points, Pixel  pixel, DotString*  buf=nullptr);
  void    draw_rect(const oat::TwoPoints&  points, Pixel  pixel, DotString*  buf=nullptr);
  void  draw_circle(const oat::TwoPoints&  points, Pixel  pixel, DotString*  buf=nullptr);


  void  fill_rect(const oat::TwoPoints&  points, Pixel  pixel, DotString*  buf=nullptr);

  void  fill_area(int  x, int  y, Pixel  pixel          , DotString*  buf=nullptr);
  void  fill_area(int  x, int  y, Pixel      a, Pixel  b, DotString*  buf=nullptr);

  void     shift_up(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void   shift_left(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void  shift_right(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void   shift_down(const oat::TwoPoints&  points, DotString*  buf=nullptr);

  void   mirror_left(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void  mirror_upper(const oat::TwoPoints&  points, DotString*  buf=nullptr);

  void   revolve_left(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void  revolve_right(const oat::TwoPoints&  points, DotString*  buf=nullptr);

  void  reverse_h(const oat::TwoPoints&  points, DotString*  buf=nullptr);
  void  reverse_v(const oat::TwoPoints&  points, DotString*  buf=nullptr);




  void  save_as_png(const char*  path, int  color_number, const oat::Color*  src_colors) const;

  void  draw(oat::Area&  area, int  x_base, int  y_base, const SDL_Rect*  rect, int  pixel_size, const Palette&  pal) const;

  void   read(const oat::Ireco&  grp);
  void  write(std::string&  buf) const;

};




class
Paper::
Palette
{
public:
  virtual void  find(Pixel  pixel){}

  virtual oat::Color  get_color(Pixel  pixel) const{return 0;}
  virtual Pixel      make_pixel(            ) const{return 0;}

};




#endif




