#include"paper.hpp"
#include<algorithm>




using namespace oat;


void
Paper::
draw_line(const TwoPoints&  points, Pixel  pixel, DotString*  buf)
{
  int  x0 = points.x0;
  int  y0 = points.y0;

  int  dx = std::abs(points.x1-points.x0);
  int  dy = std::abs(points.y1-points.y0);

  const int  sx = ((points.x0 < points.x1)? 1:-1);
  const int  sy = ((points.y0 < points.y1)? 1:-1);

  int  err = (dx-dy);

    for(;;)
    {
      put(x0,y0,pixel,buf);

        if((x0 == points.x1) && (y0 == points.y1))
        {
          break;
        }


      const int  e2 = (2*err);

        if(e2 > -dy)
        {
          err = err-dy;
          x0  = x0 +sx;
        }


        if(e2 < dx)
        {
          err = err+dx;
          y0  = y0 +sy; 
        }
    }
}


void
Paper::
draw_rect(const TwoPoints&  points, Pixel  pixel, DotString*  buf)
{
  auto  rect = make_rect(points);

    rep(xx,rect.w)
    {
      put(rect.x+xx,rect.y         ,pixel,buf);
      put(rect.x+xx,rect.y+rect.h-1,pixel,buf);
    }


    for(int  yy = 1;  yy < rect.h-1;  ++yy)
    {
      put(rect.x         ,rect.y+yy,pixel,buf);
      put(rect.x+rect.w-1,rect.y+yy,pixel,buf);
    }
}


void
Paper::
draw_circle(const TwoPoints&  points, Pixel  pixel, DotString*  buf)
{
  int  x0 = points.x0;
  int  y0 = points.y0;
  int  x1 = points.x1;
  int  y1 = points.y1;


//http://members.chello.at/~easyfilter/bresenham.htmlからほぼ丸写し
  int a = abs(x1-x0), b = abs(y1-y0), b1 = b&1; /* values of diameter */
   long dx = 4*(1-a)*b*b, dy = 4*(b1+1)*a*a; /* error increment */
   long err = dx+dy+b1*a*a, e2; /* error of 1.step */

   if (x0 > x1) { x0 = x1; x1 += a; } /* if called with swapped points */
   if (y0 > y1) y0 = y1; /* .. exchange them */
   y0 += (b+1)/2; y1 = y0-b1;   /* starting pixel */
   a *= 8*a; b1 = 8*b*b;

   do {
       put(x1, y0,pixel,buf); /*   I. Quadrant */
       put(x0, y0,pixel,buf); /*  II. Quadrant */
       put(x0, y1,pixel,buf); /* III. Quadrant */
       put(x1, y1,pixel,buf); /*  IV. Quadrant */
       e2 = 2*err;
       if (e2 <= dy) { y0++; y1--; err += dy += a; }  /* y step */ 
       if (e2 >= dx || 2*err > dy) { x0++; x1--; err += dx += b1; } /* x step */
   } while (x0 <= x1);
   
   while (y0-y1 < b) {  /* too early stop of flat ellipses a=1 */
       put(x0-1, y0,pixel,buf); /* -> finish tip of ellipse */
       put(x1+1, y0++,pixel,buf); 
       put(x0-1, y1,pixel,buf);
       put(x1+1, y1--,pixel,buf); 
   }
}


void
Paper::
fill_rect(const TwoPoints&  points, Pixel  pixel, DotString*  buf)
{
  auto  rect = make_rect(points);

    rep(yy,rect.h){
    rep(xx,rect.w){
      put(rect.x+xx,
          rect.y+yy,pixel,buf);
    }}
}




void
Paper::
fill_area(int  x, int  y, Pixel  pixel, DotString*  buf)
{
  auto  target = get_pixel(x,y);

    if(target != pixel)
    {
      put(x,y,pixel,buf);

        if(x              ){fill_area(x-1,y  ,target,pixel,buf);}
        if(y              ){fill_area(x  ,y-1,target,pixel,buf);}
        if(x < (get_w()-1)){fill_area(x+1,y  ,target,pixel,buf);}
        if(y < (get_h()-1)){fill_area(x  ,y+1,target,pixel,buf);}
    }
}


void
Paper::
fill_area(int  x, int  y, Pixel  a, Pixel  b, DotString*  buf)
{
    if(get_pixel(x,y) == a)
    {
      put(x,y,b,buf);

        if(x              ){fill_area(x-1,y  ,a,b,buf);}
        if(y              ){fill_area(x  ,y-1,a,b,buf);}
        if(x < (get_w()-1)){fill_area(x+1,y  ,a,b,buf);}
        if(y < (get_h()-1)){fill_area(x  ,y+1,a,b,buf);}
    }
}




