#include"pngbuffer.hpp"
#include<png.h>
#include<zlib.h>
#include<cstdio>




namespace oat{


enum{
  color_size = 3,
};




PNGBuffer::
PNGBuffer():
w(0),
h(0)
{
}


PNGBuffer::
PNGBuffer(PNGBuffer&&  rhs)
{
  swap(rhs);
}




void
PNGBuffer::
swap(PNGBuffer&  rhs)
{
  std::swap(color_table,rhs.color_table);
  std::swap(index_table,rhs.index_table);

  std::swap(w,rhs.w);
  std::swap(h,rhs.h);
}


int  PNGBuffer::get_width()  const{return w;}
int  PNGBuffer::get_height() const{return h;}

int  PNGBuffer::get_color_number() const{return color_table.size()/color_size;}

PNGBuffer::ColorType  PNGBuffer::get_color_type() const{return(index_table.size()? indexed_color:direct_color);}

      uint8_t*  PNGBuffer::get_index(int  x, int  y)      {return &index_table[(w*y)+x];}
const uint8_t*  PNGBuffer::get_index(int  x, int  y) const{return &index_table[(w*y)+x];}

      uint8_t*  PNGBuffer::get_color(int  x, int  y)      {return &color_table[color_size*((w*y)+x)];}
const uint8_t*  PNGBuffer::get_color(int  x, int  y) const{return &color_table[color_size*((w*y)+x)];}


void
PNGBuffer::
make_direct_color(int  _w, int  _h)
{
  w = _w;
  h = _h;

  color_table.resize(color_size*w*h);
  index_table.clear();
}


void
PNGBuffer::
make_indexed_color(int  _w, int  _h, int  color_number, const Color*  src_colors)
{
  w = _w;
  h = _h;

  color_table.resize(color_size*color_number);
  index_table.resize(w*h);

    if(src_colors)
    {
        rep(i,color_number)
        {
          auto  dst = &color_table[color_size*i];

          get_rgb(src_colors[i],dst[0],dst[1],dst[2]);
        }
    }
}


void
PNGBuffer::
convert_to(ColorType  type)
{
  PNGBuffer  tmp;

    if(type == direct_color)
    {
        if(get_color_type() == indexed_color)
        {
          tmp.make_direct_color(w,h);

            rep(y,h){
            rep(x,w){
              auto  dst = tmp.get_color(x,y);
              auto  src =     get_index(x,y);

              std::memcpy(dst,get_color(*src),color_size);
            }}


          swap(tmp);
        }
    }

  else
    if(type == indexed_color)
    {
        if(get_color_type() == direct_color)
        {
          tmp.make_indexed_color(w,h);

            rep(y,h){
            rep(x,w){
              auto  dst = tmp.get_index(x,y);

              auto  res = tmp.try_push(get_color(x,y));

                if(res >= 0)
                {
                  *dst = res;
                }
            }}


          swap(tmp);
        }
    }
}


int
PNGBuffer::
try_push(const uint8_t*  color)
{
  const int  n = (color_table.size()/color_size);

    if(n < 256)
    {
        rep(i,n)
        {
            if(std::memcmp(&color_table[color_size*i],color,color_size) == 0)
            {
              return i;
            }
        }


      push(color);

      return n;
    }


  report;

  return -1;
}


void
PNGBuffer::
push(const uint8_t*  color)
{
  color_table.push_back(color[0]);
  color_table.push_back(color[1]);
  color_table.push_back(color[2]);
}


void
PNGBuffer::
save(const char*  path)
{
  auto  f = fopen(path,"wb");

    if(!f)
    {
      return;
    }


  png_struct*  png = png_create_write_struct(PNG_LIBPNG_VER_STRING,nullptr,nullptr,nullptr);

  png_info*  png_info = png_create_info_struct(png);

  png_init_io(png,f);

  png_set_compression_level(png,Z_BEST_COMPRESSION);

  std::vector<uint8_t*>  p;

    if(get_color_type() == direct_color)
    {
      png_set_IHDR(png,png_info,w,h,8,PNG_COLOR_TYPE_RGB,PNG_FILTER_TYPE_BASE,PNG_COMPRESSION_TYPE_BASE,PNG_INTERLACE_NONE);

        rep(y,h)
        {
          p.push_back(&color_table[color_size*w*y]);
        }


      auto  pp = (uint8_t**)p.data();

      png_set_rows(png,png_info,pp);

      png_write_png(png,png_info,PNG_TRANSFORM_IDENTITY,nullptr);
    }

  else
    if(get_color_type() == indexed_color)
    {
      int      depth;
      int  transform;

      const int  color_number = (color_table.size()/color_size);

           if(color_number <=   2){  depth =  1;  transform = PNG_TRANSFORM_PACKING ;}
      else if(color_number <=   4){  depth =  2;  transform = PNG_TRANSFORM_PACKING ;}
      else if(color_number <=  16){  depth =  4;  transform = PNG_TRANSFORM_PACKING ;}
      else if(color_number <= 256){  depth =  8;  transform = PNG_TRANSFORM_IDENTITY;}
      else                        {  report;  goto QUIT;}


      std::vector<png_color>  colors(color_number);

      auto  color_src = color_table.data();

        rep(i,color_number)
        {
          colors[i].red   = *color_src++;
          colors[i].green = *color_src++;
          colors[i].blue  = *color_src++;
        }


      png_set_IHDR(png,png_info,w,h,depth,PNG_COLOR_TYPE_PALETTE,PNG_FILTER_TYPE_BASE,PNG_COMPRESSION_TYPE_BASE,PNG_INTERLACE_NONE);

      png_set_PLTE(png,png_info,colors.data(),colors.size());

        rep(y,h)
        {
          p.push_back(&index_table[w*y]);
        }


      auto  pp = (uint8_t**)p.data();

      png_set_rows(png,png_info,pp);

      png_write_png(png,png_info,transform,nullptr);
    }


QUIT:
  png_destroy_write_struct(&png,&png_info);

  fclose(f);
}


void
PNGBuffer::
load(const char*  path)
{
  auto  f = fopen(path,"wb");

    if(!f)
    {
      return;
    }


  png_struct*  png = png_create_read_struct(PNG_LIBPNG_VER_STRING,NULL,NULL,NULL);

  png_info*  png_info = png_create_info_struct(png);

    if(!png)
    {
    }


    if(!png_info)
    {
    }


  png_init_io(png,f);

  png_read_png(png,png_info,PNG_TRANSFORM_STRIP_ALPHA|PNG_TRANSFORM_PACKING,NULL);

  const int  png_w = png_get_image_width( png,png_info);
  const int  png_h = png_get_image_height(png,png_info);

  png_byte**  src = png_get_rows(png,png_info);

  int  type = png_get_color_type(png,png_info);

  uint8_t*  dst = color_table.data();

    if(type != PNG_COLOR_TYPE_PALETTE)
    {
      make_direct_color(png_w,png_h);

        rep(y,h)
        {
          const png_byte*  p = src[y];

            rep(x,w)
            {
              *dst++ = *p++;
              *dst++ = *p++;
              *dst++ = *p++;
            }
        }
    }

  else
    {
      png_color*  colors = NULL;

      int  color_number = 0;

      png_get_PLTE(png,png_info,&colors,&color_number);

      make_indexed_color(png_w,png_h,color_number);

        if(colors)
        {
            rep(i,color_number)
            {
              *dst++ = colors->red;
              *dst++ = colors->green;
              *dst++ = colors->blue;

              ++colors;
            }
        }


        rep(y,h){
        rep(x,w){
          *get_index(x,y) = src[y][x];
        }}
    }


  png_destroy_read_struct(&png,&png_info,NULL);

  fclose(f);
}




}




