#include"root.hpp"
#include"widget.d/widget.hpp"
#include"widget.d/node.hpp"
#include"pngbuffer.hpp"
#include"menu.hpp"




namespace oat{
namespace sys{


uint8_t*  main_pixels;
int        main_pitch;


namespace{


SDL_Window*      window;
SDL_Renderer*  renderer;
SDL_Texture*    texture;

SDL_Event  evt;

Node*  root;

Widget*  current_widget;

Widget*  keyreceive_widget;

Mouse
mouse;

DropCallback
drop_cb;


void
process_mouse()
{
  auto  res  = current_widget->test(mouse);

    if(!res)
    {
      current_widget->leave();

      res = root->test(mouse);
    }


    if(res)
    {
      current_widget = res;

      current_widget->enter();

      current_widget->process(mouse);
    }
}


void
start_draw()
{
  void*  ptr;

  SDL_LockTexture(texture,nullptr,&ptr,&main_pitch);

  main_pixels = (uint8_t*)ptr;
}


void
fill()
{
    rep(y,root->get_parameter().height)
    {
      auto  p = (uint32_t*)&main_pixels[main_pitch*y];

        rep(x,root->get_parameter().width)
        {
          *p++ = gray_color;
        }
    }
}


void
finish_draw()
{
  SDL_UnlockTexture(texture);


  SDL_RenderClear(renderer);

  SDL_RenderCopy(renderer,texture,nullptr,nullptr);

  SDL_RenderPresent(renderer);
}


}




void
make_window(const char*  title, int  x, int  y, Node*  _root, DropCallback  _drop_cb)
{
  window = SDL_CreateWindow(title,x,y,320,32,0);

  renderer = SDL_CreateRenderer(window,-1,0);

  texture = nullptr;

  root           = _root;
  current_widget = _root;

  current_widget->enter();

  drop_cb = _drop_cb;
}


void
clean_window()
{
  SDL_DestroyTexture(texture);
  SDL_DestroyRenderer(renderer);
  SDL_DestroyWindow(window);
}


void
set_keyreceive_widget(Widget*  wid)
{
  keyreceive_widget = wid;
}


void
save_png(const char*  path)
{
  PNGBuffer  buf;

  int  w;
  int  h;

  SDL_GetWindowSize(window,&w,&h);

  buf.make_direct_color(w,h);

  std::vector<uint8_t>  pbuf(4*w*h);

  auto  tmp_pixptr = main_pixels;
  auto  tmp_pitch  = main_pitch;

  main_pixels = pbuf.data();
  main_pitch  =       (4*w);

  fill();

  root->draw();

    rep(y,h)
    {
      auto  p = (const uint32_t*)&main_pixels[main_pitch*y];

        rep(x,w)
        {
          uint8_t  r;
          uint8_t  g;
          uint8_t  b;

          get_rgb(*p++,r,g,b);

          auto  dst = buf.get_color(x,y);

          *dst++ = r;
          *dst++ = g;
          *dst++ = b;
        }
    }


  main_pixels = tmp_pixptr;
  main_pitch  =  tmp_pitch;

  buf.save(path);

}


void
draw()
{
  start_draw();

  fill();

  root->draw();

  finish_draw();
}


void
update()
{
    if(root->test_flag(WID_ST_dirty_flag))
    {
      root->cleanup(0,0);

      const int  w = root->get_parameter().right ;
      const int  h = root->get_parameter().bottom;

      SDL_DestroyTexture(texture)                                                                         ;
                         texture = SDL_CreateTexture(renderer,colorformat,SDL_TEXTUREACCESS_STREAMING,w,h);

      SDL_SetWindowSize(window,w,h);

      draw();
    }

  else
    {
      start_draw();

      root->redraw();

      finish_draw();
    }
}




bool
process_menu_event()
{
    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_WINDOWEVENT):
              switch(evt.window.event)
              {
                case(SDL_WINDOWEVENT_EXPOSED):
                  update_menu();
                  break;
                case(SDL_WINDOWEVENT_CLOSE):
                  clean_menu();
                  break;
              }
            break;
          case(SDL_QUIT):
            return false;
            break;
          case(SDL_MOUSEBUTTONDOWN):
              if(evt.button.button == SDL_BUTTON_LEFT)
              {
                move_menu_cursor(evt.button.y);

                execute_menu_callback();
              }

            else
              {
                clean_menu();
              }
            break;
          case(SDL_MOUSEMOTION):
            move_menu_cursor(evt.motion.y);

            update_menu();
            break;
        }
    }


  return true;
}


bool
process_event()
{
    if(test_menu())
    {
      return process_menu_event();
    }


  mouse.clean();

    while(SDL_PollEvent(&evt))
    {
        switch(evt.type)
        {
          case(SDL_QUIT):
            return false;
            break;
          case(SDL_DROPFILE):
              if(drop_cb)
              {
                drop_cb(evt.drop.file);
              }


            SDL_free(evt.drop.file);
            break;
          case(SDL_MOUSEBUTTONUP  ):
          case(SDL_MOUSEBUTTONDOWN):
            mouse.button(evt.button);

            process_mouse();
            break;
          case(SDL_MOUSEMOTION):
            mouse.motion(evt.motion);

            process_mouse();
            break;
          case(SDL_KEYUP):
              if(evt.key.keysym.sym == SDLK_F5)
              {
                int  i = 0;

                char  buf[256];

                snprintf(buf,sizeof(buf),"_OAT_WINDOW%02d.png",i++);

                save_png(buf);
              }
          case(SDL_KEYDOWN):
              if(keyreceive_widget)
              {
                keyreceive_widget->process(evt.key);
              }
            break;
          case(SDL_WINDOWEVENT):
              switch(evt.window.event)
              {
                case(SDL_WINDOWEVENT_EXPOSED):
                case(SDL_WINDOWEVENT_MOVED):
                    if(texture)
                    {
                      draw();
                    }
                  break;
                case(SDL_WINDOWEVENT_CLOSE):
                  return false;
              }
            break;
          default:;
        }
    }


  current_widget->do_on_everytime();

  return true;
}


}}




