#ifndef STORAGE_HPP
#define STORAGE_HPP


#include<cstdint>
#include<string>
#include<set>
#include<deque>
#include<memory>


enum
EntryType
{
  ENT_file,
  ENT_directory,
  ENT_root_directory,
  ENT_softlink,
  ENT_hardlink,

};


class Directory;
class File;
class SoftLink;
class HardLink;
class Entry;


class
Entry
{
protected:
  std::shared_ptr<Directory>  entry_parent;

  std::string  entry_name;

public:
  const EntryType  entry_type;

  const uint32_t  entry_id;

  Entry(EntryType  type, const std::shared_ptr<Directory>&  _parent, const std::string&  _name);

  bool  operator<(const Entry&  rhs) const{return(entry_name < rhs.get_name());}
  bool  operator==(EntryType  _type) const;
  bool  operator!=(EntryType  _type) const;

  Directory&  as_directory();
  File&            as_file();
  SoftLink&    as_softlink();
  HardLink&    as_hardlink();

  const std::string  get_name() const;
  const Directory*  get_parent() const;

  std::string  to_path() const;

};


class
SoftLink: public Entry
{
  std::weak_ptr<Entry>  target;

public:
  SoftLink(const std::shared_ptr<Directory>&  _parent, const std::string&  _name, const std::shared_ptr<Entry>&  _target);

  std::shared_ptr<Entry>  get(){return target.lock();}

};


class
HardLink: public Entry
{
  std::shared_ptr<Entry>  target;

public:
  HardLink(const std::shared_ptr<Directory>&  _parent, const std::string&  _name, const std::shared_ptr<Entry>&  _target);

  const std::shared_ptr<Entry>&  get(){return target;}

};


class
File: public Entry
{
public:
  std::string  binary;

  File(const std::shared_ptr<Directory>&  _parent, const std::string&  _name);

};


class
Directory: public Entry
{

public:
  std::set<std::shared_ptr<Entry>>  entries;

  Directory(const std::shared_ptr<Directory>&  _parent, const std::string&  _name);

};




#endif




