#include"tilist.hpp"




namespace{


Paper::Frame  frame(Map::w,Map::h);


struct
{
  Tile  tiles[Map::h][Map::w];

  int  w;
  int  h;

} copy_ctx;


void
draw_map(Area&  area)
{
  area.fill(palette8.get_color(0));

    rep(y,Map::h){
    rep(x,Map::w){
      map.get_tile(x,y).draw(area,Tile::size*x,Tile::size*y,nullptr,1,palette8);
    }}


  auto  frame_rect = make_rect(*frame);

  int  x = frame_rect.x*Tile::size;
  int  y = frame_rect.y*Tile::size;
  int  w = frame_rect.w*Tile::size;
  int  h = frame_rect.h*Tile::size;

  area.draw_rect_safety(x,y,w,h,white_color,true);
}


void
touch(Area&  area, const Mouse&  mouse)
{
    if(mouse.test_left_pressing())
    {
      auto  pt = area.get_relative_point(mouse);

        if(mouse.test_left_pressed())
        {
          frame.reset();
        }


      int  x = pt.x/Tile::size;
      int  y = pt.y/Tile::size;

      frame.set(x,y);

      map.set_x_pos(x);
      map.set_y_pos(y);

      Tile::clear_undo_buffer();

      painter::reset_drawing(map.get_tile());
      painter::push_showing(map.get_tile());
      

      update_draw();
    }

  else
    if(mouse.test_right_pressing())
    {
      frame.reset();

      update_draw();
    }
}


void
copy(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  rect = make_rect(*frame);

      copy_ctx.w = rect.w;
      copy_ctx.h = rect.h;

        rep(y,rect.h){
        rep(x,rect.w){
          copy_ctx.tiles[y][x] = map.get_tile(rect.x+x,rect.y+y);
        }}
    }
}


void
paste(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  rect = make_rect(*frame);

      const int  w = ((rect.x+copy_ctx.w) >= Map::w)? (Map::w-rect.x):copy_ctx.w;
      const int  h = ((rect.y+copy_ctx.h) >= Map::h)? (Map::h-rect.y):copy_ctx.h;

        rep(y,h){
        rep(x,w){
          map.put(rect.x+x,rect.y+y,copy_ctx.tiles[y][x]);
        }}


      update_draw();
    }
}


void
tppaste(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  rect = make_rect(*frame);

      const int  w = ((rect.x+copy_ctx.w) >= Map::w)? (Map::w-rect.x):copy_ctx.w;
      const int  h = ((rect.y+copy_ctx.h) >= Map::h)? (Map::h-rect.y):copy_ctx.h;

        rep(y,h){
        rep(x,w){
          map.put(rect.x+x,rect.y+y,copy_ctx.tiles[y][x],true);
        }}


      update_draw();
    }
}


void
save(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  s = map.widgets.filename_txtbox->get();

        if(s.find(".tlmp") == std::string::npos)
        {
          s += ".tlmp";
        }


      map.save(s.data());
    }
}


void
save_as_png(Button&  btn)
{
    if(btn->test_pressed())
    {
      auto  s = map.widgets.filename_txtbox->get();

      auto  s2 = s;

      s  += ".png";
      s2 += "-map.png";

       map.save_as_png(s.data());
      rmap.save_as_png(s2.data());
    }
}




void
callback_with_rect(Button&  btn, void  (Map::*fn)(const SDL_Rect&), bool  repeating=false)
{
    if(btn->test_pressing())
    {
      (map.*fn)(make_rect(*frame));

      update_draw();
    }
}


void       shift_up(Button&  btn){callback_with_rect(btn,&Map::shift_up,true);}
void     shift_left(Button&  btn){callback_with_rect(btn,&Map::shift_left,true);}
void    shift_right(Button&  btn){callback_with_rect(btn,&Map::shift_right,true);}
void     shift_down(Button&  btn){callback_with_rect(btn,&Map::shift_down,true);}




Widget*
create_lrshift_widget()
{
  auto  row = new TableRow;

  auto  lb = new Label(u"左右シフト");

  auto  l_btn = new Button(default_left_icon);
  auto  r_btn = new Button(default_right_icon);

  l_btn->set_callback(shift_left,true);
  r_btn->set_callback(shift_right,true);

  row->insert(lb);
  row->insert(l_btn);
  row->insert(r_btn);

  return row;
}


Widget*
create_udshift_widget()
{
  auto  row = new TableRow;

  auto  lb = new Label(u"上下シフト");

  auto  l_btn = new Button(default_up_icon);
  auto  r_btn = new Button(default_down_icon);

  l_btn->set_callback(shift_up,true);
  r_btn->set_callback(shift_down,true);

  row->insert(lb);
  row->insert(l_btn);
  row->insert(r_btn);

  return row;
}


}


void
Map::Widgets::
initialize()
{
  area = new Area(Tile::size*Map::w,
                  Tile::size*Map::h);

  area->set_draw_callback(draw_map);
  area->set_callback(touch);


  saving_node = new TableColumn;

  filename_txtbox = new TextBox("newdata",32);

  auto  save_btn        = new RotorButton(u"save");
  auto  save_as_png_btn = new RotorButton(u"save as png");


  save_btn->set_callback(::save);
  save_as_png_btn->set_callback(::save_as_png);

  saving_node->insert(filename_txtbox);
  saving_node->insert(save_btn);
  saving_node->insert(save_as_png_btn);


  ud_shift_widget = create_udshift_widget();
  lr_shift_widget = create_lrshift_widget();


   copy_button  = new RotorButton(u"コピー");
  paste_button = new      Button(u"貼り付け");
  tppaste_button = new      Button(u"透過貼り付け");


  copy_button->set_callback(::copy);
  tppaste_button->set_callback(::tppaste);
  paste_button->set_callback(::paste);
}




