#include"util.hpp"
#include<zlib.h>




namespace oat{


int
fputc16le(int  c, FILE*  f)
{
  uint16_t  v = c;

  fputc(v   ,f);
  fputc(v>>8,f);

  return v;
}


int
fputc32le(int  c, FILE*  f)
{
  uint32_t  v = c;

  fputc(v    ,f);
  fputc(v>> 8,f);
  fputc(v>>16,f);
  fputc(v>>24,f);

  return v;
}


int
fgetc16le(FILE*  f)
{
  uint16_t  v  = fgetc(f)   ;
            v |= fgetc(f)<<8;

  return(int16_t)v;
}


int
fgetc32le(FILE*  f)
{
  uint32_t  v  = fgetc(f)    ;
            v |= fgetc(f)<< 8;
            v |= fgetc(f)<<16;
            v |= fgetc(f)<<24;

  return(int32_t)v;
}




void
save_file(const void*  src_data, size_t  src_size, const char*  path)
{
  FILE*  f = fopen(path,"wb");

    if(!f)
    {
      report;
      return;
    }


    if(fwrite(src_data,1,src_size,f) != src_size)
    {
      report;
    }


  fclose(f);
}



void
save_gzfile(const void*  src_data, size_t  src_size, const char*  path)
{
  gzFile  gz = gzopen(path,"wb9");

    if(!gz)
    {
      report;
      return;
    }


    if(gzwrite(gz,(const unsigned char*)src_data,src_size) != src_size)
    {
      report;
    }


  gzclose(gz);
}



std::vector<uint8_t>
load_file(const char*  path)
{
  std::vector<uint8_t>  s;

  FILE*  f = fopen(path,"rb");

    if(!f)
    {
      report;
      return std::move(s);
    }


  
    for(;;)
    {
      const int  c = fgetc(f);

        if(feof(f))
        {
          break;
        }


      s.push_back(c);
    }


  fclose(f);

  return std::move(s);
}


std::vector<uint8_t>
load_gzfile(const char*  path)
{
  std::vector<uint8_t>  s;

  gzFile  gz = gzopen(path,"rb");

    if(!gz)
    {
      return load_file(path);
    }


  
    for(;;)
    {
      const int  c = gzgetc(gz);

        if(gzeof(gz))
        {
          break;
        }


      s.push_back(c);
    }


  gzclose(gz);

  return std::move(s);
}




}




