#ifndef OAT_UTIL_RWOPS_HPP
#define OAT_UTIL_RWOPS_HPP


#include<cstdint>
#include<cstdio>
#include<cstdlib>
#include<memory>
#include<SDL.h>


namespace oat{


class
rwops
{
  SDL_RWops*  ctx;

public:
  rwops(SDL_RWops*  _ctx): ctx(_ctx){}

  ~rwops(){close();}

  void  close(){SDL_RWclose(ctx);}

  void  put(char  c)
  {
    SDL_RWwrite(ctx,&c,1,1);
  }

  void  put_le16(int  c){SDL_WriteLE16(ctx,c);}
  void  put_le32(int  c){SDL_WriteLE32(ctx,c);}

  int  get()
  {
    char  c;

    SDL_RWread(ctx,&c,1,1);

    return c;
  }


  int  get_le16(){SDL_ReadLE16(ctx);}
  int  get_le32(){SDL_ReadLE32(ctx);}

  void  seek(int  n){SDL_RWseek(ctx,n,RW_SEEK_CUR);}

  int  setpos(int  n){SDL_RWseek(ctx,n,RW_SEEK_SET);}
  int  getpos(      ){SDL_RWtell(ctx);}

  int  size(){return SDL_RWsize(ctx);}

  static std::unique_ptr<rwops>  open_rw(const char*  path)
  {
    auto  ctx = SDL_RWFromFile(path,"wb");

    std::unique_ptr<rwops>  uniptr(ctx? new rwops(ctx):nullptr);

    return std::move(uniptr);
  }

  static std::unique_ptr<rwops>  open_ro(const char*  path)
  {
    auto  ctx = SDL_RWFromFile(path,"rb");

    std::unique_ptr<rwops>  uniptr(ctx? new rwops(ctx):nullptr);

    return std::move(uniptr);
  }

};


}




#endif




