#include"button.hpp"




namespace oat{


namespace{


uint32_t  last_ticks;

int  repeat_interval;
int  repeat_count;


enum{
  freezed_flag   = 0x2,
  repeating_flag = 0x4,
  toggle_flag    = 0x8,

};


}




Button::
Button(const char16_t*  text):
elements({Element(text)}),
button_state(0)
{
  widget_parameter.height = Icon::size+8;

  reset_size();
}


Button::
Button(const Icon&  icon):
elements({Element(icon)}),
button_state(0)
{
  widget_parameter.height = Icon::size+8;

  reset_size();
}


Button::
Button(std::initializer_list<Element>  ls):
elements(ls),
button_state(0)
{
  widget_parameter.height = Icon::size+8;

  reset_size();
}




const ButtonParameter*
Button::
operator->() const
{
  return &button_parameter;
}


void
Button::
set_toggle()
{
  button_state |= toggle_flag;
}


void
Button::
set_callback(Callback  _cb, bool  repeat)
{
  cb = _cb;

  button_state &= ~repeating_flag;

    if(repeat)
    {
      button_state |= repeating_flag;
    }
}


void
Button::
reset(std::initializer_list<Element>  ls)
{
  elements = ls;


  reset_size();

  request_redraw();
}


void
Button::
reset_size()
{
  const int  prev_w = widget_parameter.width;

  widget_parameter.width = 4;

    for(auto&  ele: elements)
    {
        switch(ele.type)
        {
          case(ELE_text): widget_parameter.width += font::get_width(ele.text);break;
          case(ELE_icon): widget_parameter.width += Icon::size               ;break;
        }


      widget_parameter.width += 4;
    }


    if(prev_w != widget_parameter.width)
    {
      set_flag(WID_ST_dirty_flag,true);
    }
}


void
Button::
reset_repeat()
{
  repeat_interval = 0;
}



void
Button::
freeze()
{
  button_state |= freezed_flag;

  request_redraw();
}


void
Button::
unfreeze()
{
  button_state &= ~freezed_flag;

  request_redraw();
}


bool
Button::
test_freezing() const
{
  return(button_state&freezed_flag);
}


bool
Button::
test_repeating() const
{
  return repeat_interval;
}


bool
Button::
test_toggle() const
{
  return(button_state&toggle_flag);
}


void
Button::
process(Mouse&  mouse)
{
    if(test_freezing())
    {
      goto QUIT;
    }


  button_parameter.clean();

    if(!button_parameter.test_touching())
    {
        if(mouse.test_left_pressing())
        {
          goto QUIT;
        }
    }


  button_parameter.touch();

    if(test_toggle())
    {
        if(mouse.test_left_unpressed())
        {
            if(button_parameter.test_pressing())
            {
              button_parameter.unpress();
            }

          else
            {
              button_parameter.press();
            }


            if(cb)
            {
              cb(*this);
            }
        }
    }

  else
    {
        if(mouse.test_left_pressing())
        {
          button_parameter.press();
        }

      else
        {
          button_parameter.unpress();

          reset_repeat();
        }


        if(cb)
        {
          cb(*this);
        }


        if(!mouse.test_left_pressing())
        {
          reset_repeat();
        }
    }


QUIT:
  request_redraw();
}


void
Button::
do_on_leave()
{
    if(!test_freezing() && !test_toggle())
    {
      button_parameter.unpress();
      button_parameter.untouch();

      repeat_interval = 0;

      request_redraw();
    }
}


void
Button::
do_on_everytime()
{
    if((button_state&repeating_flag) && button_parameter.test_pressing())
    {
      auto  now = SDL_GetTicks();

        if(repeat_interval)
        {
            if(now >= (last_ticks+repeat_interval))
            {
              last_ticks = now;

                if(cb)
                {
                  cb(*this);

                    if(++repeat_count > 6)
                    {
                      repeat_count = 0;

                      repeat_interval /= 2;
                      repeat_interval |= 1;
                    }
                }
            }
        }

      else
        {
            if(now >= last_ticks+1000)
            {
              last_ticks = now;

              repeat_interval = 250;

              repeat_count = 0;
            }
        }
    }
}




void
Button::
draw()
{
    if(button_parameter.test_pressing()){draw_sank_button(0,0,widget_parameter.width,widget_parameter.height);}
  else                                  {     draw_button(0,0,widget_parameter.width,widget_parameter.height);}


  int  x = 4;
  int  y = 4;

    for(auto&  ele: elements)
    {
        switch(ele.type)
        {
          case(ELE_text):
            draw_text(x,y+4,black_color,ele.text);

              if(test_freezing())
              {
                draw_text(x+1,y+4+1,get_default_color(0xDF),ele.text);
              }
            break;
          case(ELE_icon):
            Widget::draw(x,y,*ele.icon);

            x += Icon::size;
            break;
        }


      x += 4;
    }
}




}




