#ifndef OAT_BUTTON_HPP
#define OAT_BUTTON_HPP


#include"widget.hpp"
#include"font.hpp"
#include"button_parameter.hpp"
#include<vector>
#include<initializer_list>


namespace oat{


class
Button: public Widget
{
public:
  using Callback = void  (*)(Button&  btn);

  enum ElementType{
    ELE_icon,
    ELE_text,
  };


  struct Element
  {
    ElementType  type;

    union{
      const Icon*      icon;
      const char16_t*  text;
    };


    Element(const char16_t*  _text): type(ELE_text), text( _text){}
    Element(const Icon&      _icon): type(ELE_icon), icon(&_icon){}
  };


  using Elements = std::vector<Element>;

protected:
  Elements  elements;

  ButtonParameter  button_parameter;

  uint32_t  button_state;

  Callback  cb=nullptr;


  void  reset_repeat();

public:
  enum{
    size = font::h+8,
  };


  Button(const char16_t*  text);
  Button(const Icon&      icon);
  Button(std::initializer_list<Element>  ls);


  const ButtonParameter*  operator->() const;

  bool  test_freezing() const;
  bool  test_repeating() const;
  bool  test_toggle() const;

  void  set_toggle();

  void  set_callback(Callback  _cb, bool  repeat=false);

  void  reset(std::initializer_list<Element>  ls);
  void  reset_size();

  void    freeze();
  void  unfreeze();

  void  process(Mouse&  mouse);

  void  do_on_leave();
  void  do_on_everytime();

  void  draw();

};




}




#endif




