#include"counter.hpp"




namespace oat{


namespace{


void
up_cb(Button&  btn)
{
  auto&  cntr = *(Counter*)btn.get_userdata();

    if(btn.test_repeating() || btn->test_unpressed())
    {
      cntr.up();
    }
}


void
down_cb(Button&  btn)
{
  auto&  cntr = *(Counter*)btn.get_userdata();

    if(btn.test_repeating() || btn->test_unpressed())
    {
      cntr.down();
    }
}


}




void
Counter::
initialize(Type  _type, int  _min, int  _max, int  _step)
{
  type = _type;

  min  =  _min;
  cur  =  _min;
  step = _step;

  cb = nullptr;

  down_button = new Button((type == T_left_and_right)? default_left_icon:default_down_icon);

  down_button->set_userdata(this);

  down_button->set_callback(down_cb,true);

  cur_label = new Label(u"-????");
  max_label = new Label(u"/????");

  cur_label->set_back_color(0xFFFFFFFF);
  max_label->set_back_color(0xFFFFFFFF);


  up_button = new Button((type == T_left_and_right)?  default_right_icon:default_up_icon);

  up_button->set_userdata(this);

  up_button->set_callback(up_cb,true);

  int  x = 0;
  int  y = 0;

    switch(type)
    {
      case(T_left_and_right):
        insert(cur_label,x,0);

        x += font::small_w*5;

        insert(max_label,x,0);

        x += font::small_w*6;

        insert(down_button,x,0);

        x += 30;

        insert(up_button,x,0);
        break;
      case(T_up_and_down):
        insert(cur_label,0,12);

        x += font::small_w*5;

        insert(max_label,x,12);

        x += font::small_w*6;

        insert(up_button,x,y);

        y += 24;

        insert(down_button,x,y);
        break;
    }


  refresh();

  set_max(_max);
}




Counter::
Counter(int  _min, int  _max, int  _step)
{
  initialize(T_left_and_right,_min,_max,_step);
}


Counter::
Counter(Type  _type, int  _min, int  _max, int  _step)
{
  initialize(_type,_min,_max,_step);
}




Counter::
operator  int() const
{
  return cur;
}


void  Counter::set_userdata(void*  ptr, bool  spread){Widget::set_userdata(ptr);}


const Button&  Counter::get_up_button()   const{return   *up_button;}
const Button&  Counter::get_down_button() const{return *down_button;}


void
Counter::
set_callback(Callback  _cb)
{
  cb = _cb;
}


const int&  Counter::get_min() const{return min;}
const int&  Counter::get_cur() const{return cur;}
const int&  Counter::get_max() const{return max;}


void
Counter::
set_min(int  _min)
{
  min = _min;
}


void
Counter::
set_cur(int  _cur, bool  use_cb)
{
  int  old = cur       ;
             cur = _cur;

  refresh();

    if(use_cb && cb)
    {
      cb(*this,cur,old);
    }
}


void
Counter::
set_max(int  _max)
{
  max = _max;

  max_label->printf("/%4d",max);
}




void
Counter::
refresh()
{
  cur_label->printf("%4d",cur);
}




void
Counter::
set_looping_flag(bool  v)
{
  looping_flag = v;
}




void
Counter::
up()
{
  int  old = cur        ;
             cur += step;

    if(looping_flag)
    {
        while(cur > max)
        {
          cur -= max;
        }
    }

  else
    {
      cur = std::min(cur,max);
    }


    if(cb)
    {
      cb(*this,cur,old);
    }


  refresh();
}


void
Counter::
down()
{
  int  old = cur        ;
             cur -= step;

    if(looping_flag)
    {
        while(cur < min)
        {
          cur += max;
        }
    }

  else
    {
      cur = std::max(cur,min);
    }


    if(cb)
    {
      cb(*this,cur,old);
    }


  refresh();
}




}




