#include"node.hpp"




namespace oat{




Node::
Node():
Widget(true)
{
}


Node::
~Node()
{
    for(auto  ptr: children)
    {
      delete ptr;
    }
}




void
Node::
set_userdata(void*  ptr, bool  spread)
{
  Widget::set_userdata(ptr);

    if(spread)
    {
        for(auto&  it: children)
        {
          it->set_userdata(ptr,true);
        }
    }
}


void
Node::
cleanup(int  x, int  y)
{
  widget_parameter.width  = 0;
  widget_parameter.height = 0;
  widget_parameter.right  = 0;
  widget_parameter.bottom = 0;

  widget_parameter.update(x,y);

    for(auto&  it: children)
    {
        if(it->test_flag(WID_ST_dirty_flag))
        {
          it->cleanup(widget_parameter.left,widget_parameter.top);

          widget_parameter.right  = std::max(widget_parameter.right ,it->get_parameter().right );
          widget_parameter.bottom = std::max(widget_parameter.bottom,it->get_parameter().bottom);
        }
    }


  widget_parameter.unmargined_right  = (widget_parameter.right -widget_parameter.right_margin );
  widget_parameter.unmargined_bottom = (widget_parameter.bottom-widget_parameter.bottom_margin);

  widget_parameter.width  = (widget_parameter.unmargined_right -widget_parameter.margined_left);
  widget_parameter.height = (widget_parameter.unmargined_bottom-widget_parameter.margined_top );

  reset_flag(WID_ST_dirty_flag);
}


void
Node::
insert(Widget*  wid, int  x, int  y)
{
    if(wid)
    {
      children.emplace_back(wid);

      wid->join_to(*this,x,y);
    }
}




Widget*
Node::
test(const Mouse&  mouse)
{
  auto  res = Widget::test(mouse);

    if(res)
    {
        for(auto  ptr: children)
        {
          res = ptr->test(mouse);

            if(res)
            {
              break;
            }
        }
    }


  return res;
}




void
Node::
draw()
{
    if(test_flag(WID_ST_visible_flag))
    {
        for(auto&  it: children)
        {
            if(it->test_flag(WID_ST_visible_flag))
            {
              it->draw();
            }
        }
    }
}


void
Node::
redraw()
{
    if(test_flag(WID_ST_redraw_flag))
    {
        if(test_flag(WID_ST_visible_flag))
        {
            for(auto&  it: children)
            {
                if(it->test_flag(WID_ST_fill_flag))
                {
                  it->fill();

                  it->reset_flag(WID_ST_fill_flag);
                }


              it->redraw();
            }
        }


      reset_flag(WID_ST_redraw_flag);
    }
}




}




