#include"widget.hpp"
#include"node.hpp"
#include"../root.hpp"
#include<cstring>




namespace oat{




namespace{


void*  default_userdata;


}




void   Widget::set_default_userdata(void*  ptr){       default_userdata = ptr;}
void*  Widget::get_default_userdata(          ){return default_userdata      ;}




Widget::
Widget(int  _node_flag):
node_flag(_node_flag),
parent(nullptr),
widget_flags(WID_ST_active_flag|WID_ST_visible_flag|WID_ST_dirty_flag),
userdata(default_userdata)
{
  fore_color = 0xFFFFFFFF;
  back_color = gray_color;
}


Widget::
~Widget()
{
}




Node*  Widget::get_parent(){return parent;}


void   Widget::set_userdata(void*  data, bool  spread){       userdata = data;}
void*  Widget::get_userdata(                         ){return userdata       ;}


void
Widget::
cleanup(int  x, int  y)
{
  widget_parameter.update(x,y);

  reset_flag(WID_ST_dirty_flag);
}


void
Widget::
join_to(Node&  _parent, int  x, int  y)
{
    if(!parent)
    {
      parent = &_parent;

      widget_parameter.rel_point.x = x;
      widget_parameter.rel_point.y = y;

      set_flag(WID_ST_dirty_flag);
    }
}


bool  Widget::test_flag(uint32_t  flag) const{return(widget_flags&flag);}


void
Widget::
set_flag(uint32_t  flag, bool  notify)
{
  widget_flags |= flag;

    if(notify && parent)
    {
      parent->set_flag(flag,true);
    }
}


void  Widget::reset_flag(uint32_t  flag){widget_flags &= ~flag;}


const WidgetParameter&
Widget::
get_parameter() const
{
  return widget_parameter;
}


const WidgetParameter*
Widget::
operator->() const
{
  return &widget_parameter;
}


void  Widget::set_fore_color(Color  color){fore_color = color;}
void  Widget::set_back_color(Color  color){back_color = color;}


Widget*
Widget::
test(const Mouse&  mouse)
{
    if(test_flag(WID_ST_active_flag) &&
       (mouse.get_abs_x() >=   widget_parameter.left) &&
       (mouse.get_abs_y() >=    widget_parameter.top) &&
       (mouse.get_abs_x() <   widget_parameter.right) &&
       (mouse.get_abs_y() <  widget_parameter.bottom))
    {
      return this;
    }


  return nullptr;
}


SDL_Point
Widget::
get_relative_point(const Mouse&  mouse) const
{
  SDL_Point  pt;

       if(mouse.get_abs_x() <    widget_parameter.margined_left ){pt.x = 0;}
  else if(mouse.get_abs_x() >= widget_parameter.unmargined_right){pt.x = (widget_parameter.width-1);}
  else                                                           {pt.x = mouse.get_abs_x()-widget_parameter.margined_left;}

       if(mouse.get_abs_y() <    widget_parameter.margined_top   ){pt.y =  0;}
  else if(mouse.get_abs_y() >= widget_parameter.unmargined_bottom){pt.y = (widget_parameter.height-1);}
  else                                                            {pt.y = mouse.get_abs_y()-widget_parameter.margined_top;}


  return pt;
}




void
Widget::
redraw()
{
    if(test_flag(WID_ST_redraw_flag))
    {
        if(test_flag(WID_ST_visible_flag))
        {
          draw();
        }


      reset_flag(WID_ST_redraw_flag);
    }
}


void
Widget::
request_redraw()
{
  set_flag(WID_ST_redraw_flag,true);
}


void
Widget::
enter()
{
  set_flag(WID_ST_current_flag);

  SDL_ShowCursor(test_flag(WID_ST_hide_cursor_flag)? 0:1);

  do_on_enter();
}


void
Widget::
leave()
{
  reset_flag(WID_ST_current_flag);

  do_on_leave();
}


void
Widget::
enable()
{
  set_flag(WID_ST_active_flag);
  set_flag(WID_ST_visible_flag);

  request_redraw();
}


void
Widget::
disable()
{
  reset_flag(WID_ST_active_flag);
  reset_flag(WID_ST_visible_flag);

  request_redraw();


  set_flag(WID_ST_fill_flag);
}




void
Widget::
print() const
{
  widget_parameter.print();
}




}




