#ifndef OAT_WIDGET_HPP
#define OAT_WIDGET_HPP


#include"widget_parameter.hpp"
#include"../mouse.hpp"
#include"../color.hpp"
#include"../font.hpp"
#include"../util.hpp"
#include"../icon.hpp"
#include<list>


namespace oat{



class Node;


enum{
  WID_ST_active_flag        = 0x01,
  WID_ST_visible_flag       = 0x02,
  WID_ST_hide_cursor_flag   = 0x04,
  WID_ST_redraw_flag        = 0x08,
  WID_ST_fill_flag          = 0x10,
  WID_ST_dirty_flag         = 0x20,
  WID_ST_current_flag       = 0x40,

};


class
Widget
{
  void*  userdata;

protected:
  Node*  parent;

  WidgetParameter  widget_parameter;

  uint32_t  widget_flags;

  Color  fore_color;
  Color  back_color;

public:
  std::string  widget_name;

  static void   set_default_userdata(void*  ptr);
  static void*  get_default_userdata(          );

  const int  node_flag;

           Widget(int  _node_flag=false);
  virtual ~Widget(                     );


  Node*  get_parent();

  void  join_to(Node&  _parent, int  x, int  y);

  bool  test_flag(uint32_t  flag) const;

  void    set_flag(uint32_t  flag, bool  notify=false);
  void  reset_flag(uint32_t  flag                    );


  virtual Widget*  test(const Mouse&  mouse);

  SDL_Point  get_relative_point(const Mouse&  mouse) const;


  virtual void   set_userdata(void*  data, bool  spread=false);
          void*  get_userdata(                               );

  virtual void  cleanup(int  x, int  y);

  const WidgetParameter&  get_parameter() const;

  const WidgetParameter*  operator->() const;

  void  request_redraw();

  void   set_fore_color(Color  color);
  void   set_back_color(Color  color);


  virtual void    draw(){};
  virtual void  redraw();

  void  fill(            );
  void  fill(Color  color);



  virtual void  process(Mouse&  mouse){};
  virtual void  process(const SDL_KeyboardEvent&  evt){};

  virtual void  do_on_everytime(){};
  virtual void  do_on_enter(){request_redraw();};
  virtual void  do_on_leave(){request_redraw();};


  void  enter();
  void  leave();

  void   enable();
  void  disable();




  uint8_t*  get_ptr(int  x, int  y);

  void  draw_dot(       int  x, int  y, Color  color);
  void  draw_dot_safety(int  x, int  y, Color  color);

  void  draw_text(int  _x, int  _y, Color  color, const std::string&      u8s);
  void  draw_text(int  _x, int  _y, Color  color, const std::u16string&  u16s);

  void  draw(int  _x, int  _y, const Icon&  icon);

  void  draw_vline(int  x, int  y, int  w, Color  color, bool  brok=false);
  void  draw_hline(int  x, int  y, int  w, Color  color, bool  brok=false);

  void  draw_vline_safety(int  x, int  y, int  w, Color  color, bool  brok=false);
  void  draw_hline_safety(int  x, int  y, int  w, Color  color, bool  brok=false);

  void  draw_rect(int  x, int  y, int  w, int  h, Color  color, bool  brok=false);
  void  fill_rect(int  x, int  y, int  w, int  h, Color  color);

  void  draw_rect_safety(int  x, int  y, int  w, int  h, Color  color, bool  brok=false);

  void       draw_button(int  x, int  y, int  w, int  h);
  void  draw_sank_button(int  x, int  y, int  w, int  h);

  void  draw_input_form(int  x, int  y, int  w, int  h);

  virtual void  print() const;

};


class
WidgetDefaultUserDataSetter
{
public:
  WidgetDefaultUserDataSetter(void*  ptr){Widget::set_default_userdata(ptr);}

};


}




#endif




