#include"widget.hpp"
#include"../root.hpp"




namespace oat{




void
Widget::
fill()
{
  fill(back_color);
}


void
Widget::
fill(Color  color)
{
  auto  base = &sys::main_pixels[sys::main_pitch*widget_parameter.margined_top];

  base += 4*widget_parameter.margined_left;

    rep(y,widget_parameter.height)
    {
      auto  p = (uint32_t*)base                   ;
                           base += sys::main_pitch;

        rep(x,widget_parameter.width)
        {
          *p++ = color;
        }
    }
}


uint8_t*
Widget::
get_ptr(int  x, int  y)
{
  x += widget_parameter.margined_left;
  y += widget_parameter.margined_top ;

  return &sys::main_pixels[(sys::main_pitch*y)+(4*x)];
}


void
Widget::
draw_dot(int  x, int  y, Color  color)
{
  *reinterpret_cast<uint32_t*>(get_ptr(x,y)) = color;
}


void
Widget::
draw_dot_safety(int  x, int  y, Color  color)
{
    if((x >=                       0) &&
       (x <  widget_parameter.width ) &&
       (y >=                       0) &&
       (y <  widget_parameter.height))
    {
      draw_dot(x,y,color);
    }
}


void
Widget::
draw_text(int  _x, int  _y, Color  color, const std::string&  u8s)
{
  _x += widget_parameter.margined_left;
  _y += widget_parameter.margined_top ;

  font::draw_ascii(u8s.data(),sys::main_pixels,sys::main_pitch,_x,_y,color);
}


void
Widget::
draw_text(int  _x, int  _y, Color  color, const std::u16string&  u16s)
{
  _x += widget_parameter.margined_left;
  _y += widget_parameter.margined_top ;

  font::draw_unicode(u16s.data(),sys::main_pixels,sys::main_pitch,_x,_y,color);
}


void
Widget::
draw(int  _x, int  _y, const Icon&  icon)
{
    rep(yy,Icon::size){
    rep(xx,Icon::size){
      auto  code = icon.data[yy][xx];

        if(!(code&0x8000))
        {
          draw_dot(_x+xx,_y+yy,default_colors[code]);
        }
    }}
}


void
Widget::
draw_vline(int  x, int  y, int  w, Color  color, bool  brok)
{
    if(!brok)
    {
        while(w--)
        {
          draw_dot(x,y++,color);
        }
    }

  else
    {
      w /= 2;

        while(w--)
        {
          draw_dot(x,y,color);

          y += 2;
        }
    }
}


void
Widget::
draw_hline(int  x, int  y, int  w, Color  color, bool  brok)
{
    if(!brok)
    {
        while(w--)
        {
          draw_dot(x++,y,color);
        }
    }

  else
    {
      w /= 2;

        while(w--)
        {
          draw_dot(x,y,color);

          x += 2;
        }
    }
}




void
Widget::
draw_vline_safety(int  x, int  y, int  w, Color  color, bool  brok)
{
    if((x >=                     0) &&
       (x < widget_parameter.width))
    {
        if(y < 0)
        {
            if(w < -y)
            {
              return;
            }


          w += y    ;
               y = 0;
        }


        if((y+w) >= widget_parameter.height)
        {
          w = (widget_parameter.height-y);
        }


      draw_vline(x,y,w,color,brok);
    }
}


void
Widget::
draw_hline_safety(int  x, int  y, int  w, Color  color, bool  brok)
{
    if((y >=                      0) &&
       (y < widget_parameter.height))
    {
        if(x < 0)
        {
            if(w < -x)
            {
              return;
            }


          w += x    ;
               x = 0;
        }


        if((x+w) >= widget_parameter.width)
        {
          w = (widget_parameter.width-x);
        }


      draw_hline(x,y,w,color,brok);
    }
}




void
Widget::
draw_rect(int  x, int  y, int  w, int  h, Color  color, bool  brok)
{
  draw_vline(x    ,y,h,color,brok);
  draw_vline(x+w-1,y,h,color,brok);

  draw_hline(x,y    ,w,color,brok);
  draw_hline(x,y+h-1,w,color,brok);
}


void
Widget::
fill_rect(int  x, int  y, int  w, int  h, Color  color)
{
  x += widget_parameter.margined_left;
  y += widget_parameter.margined_top ;

  auto  p8 = &sys::main_pixels[(sys::main_pitch*y)+(4*x)];

    rep(yy,h)
    {
      auto  p = (uint32_t*)p8                   ;
                           p8 += sys::main_pitch;

        rep(xx,w)
        {
          *p++ = color;
        }
    }
}




void
Widget::
draw_rect_safety(int  x, int  y, int  w, int  h, Color  color, bool  brok)
{
  draw_vline_safety(x    ,y,h,color,brok);
  draw_vline_safety(x+w-1,y,h,color,brok);

  draw_hline_safety(x,y    ,w,color,brok);
  draw_hline_safety(x,y+h-1,w,color,brok);
}




void
Widget::
draw_button(int  x, int  y, int  w, int  h)
{
  fill_rect(x,y,w,h,gray_color);

  draw_rect(x,y  ,w,  2,white_color);
  draw_rect(x,y+2,2,h-2,white_color);

  draw_rect(x    ,y+h-2,w,  2,dark_gray_color);
  draw_rect(x+w-2,y+2  ,2,h-2,dark_gray_color);
}


void
Widget::
draw_sank_button(int  x, int  y, int  w, int  h)
{
  fill_rect(x,y,w,h,dark_gray_color);

  draw_rect(x  ,y  ,w  ,h  ,dark_gray_color);
  draw_rect(x+1,y+1,w-2,h-2,dark_gray_color);

  draw_vline(x+w-1,y+  1,h-2,white_color);
  draw_hline(x+  1,y+h-1,w-2,white_color);
}
 

void
Widget::
draw_input_form(int  x, int  y, int  w, int  h)
{
  fill_rect(x+1,y+1,w-2,h-2,white_color);

  draw_hline(0  ,0  ,w,black_color);
  draw_vline(0  ,0  ,h,black_color);
  draw_hline(0  ,h-1,w,white_color);
  draw_vline(w-1,0  ,h,white_color);
}




}




